using System;
using System.Xml;
using System.Collections;

namespace ChessApplication
{
	/// <summary>
	/// Search for moves using Alpha Beta pruning.
	/// </summary>
	public class AlphaBetaEngine : ChessEngine
	{
		/// <summary>
		/// Constructor of a SimpleEngine.
		/// </summary>
		/// <param name="board"> The board to generate moves for.</param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		public AlphaBetaEngine(ChessBoard board, Stack boardHistoryStack)
			: base(board,boardHistoryStack,4)
		{ 
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove); 
		}
		/// <summary>
		/// Constructor of a SimpleEngine.
		/// </summary>
		/// <param name="board"> The board to generate moves for</param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		/// <param name="searchDepth"> The depth of the search</param>
		public AlphaBetaEngine(ChessBoard board, Stack boardHistoryStack, int searchDepth)
			: base(board,boardHistoryStack, searchDepth) 
		{ 
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove); 
		}
		/// <summary>
		/// Deserialization constructor of a ChessEngine.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		public AlphaBetaEngine(XmlReader reader, ChessBoard board, Stack boardHistoryStack) 
			: base(reader,board,boardHistoryStack) 
		{ 
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove); 
		}
		/// <summary>
		/// The approximate number of nodes the have to be searched
		/// to decide the best move.
		/// </summary>
		/// <returns> Returns the number of nodes that will be searched</returns>
		public override int ApproxNrNodes()
		{
			return (int)Math.Pow(35,3.0*searchDepth/4);
		}
		/// <summary>
		/// Decides the best move and returns it.
		/// </summary>
		/// <returns> The best move using this engine</returns>
		public override ChessMove DecideBestMove()
		{
			double /*alpha=-2000,*/ beta=2000;
			ChessMove[] moves;
			ChessBoard[] boards;
			double[] values;
			double bestValue=-2000;
			double currentValue;
			ChessMove bestMove = null;
			nrGeneratedMoves = 0;
			forBoard.GenerateSucessor(out moves, out boards);
			Array.Sort(moves,boards);
			ShuffleInClasses(moves,boards);
			nrGeneratedMoves += moves.Length;
			values = new double[boards.Length];
			boardHistoryStack.Push(forBoard);
			for(int i=0; i<boards.Length; ++i)
			{
				//currentValue = MinValue(boards[i],alpha,beta,1);
				currentValue = MinValue(boards[i],bestValue,beta,1);
				if (currentValue > bestValue)
				{
					bestValue = currentValue;
					bestMove = moves[i];
				}
				//values[i] = MinValue(boards[i],alpha,beta,1);
			}
			//Array.Sort(values,moves);
			nrGeneratedMoves = 0;
			boardHistoryStack.Pop();
			if (bestValue<0.0)
			{
				if (forBoard.Draw50MovesRule())
					return ChessMove.ClaimDrawRule50Move();
				if (forBoard.DrawMoveRepetition(boardHistoryStack))
					return ChessMove.ClaimDrawRuleRepetition();
			}
			return bestMove;
			//return moves[values.Length-1];
		}

		private double MaxValue(ChessBoard board, double alpha, double beta, int level)
		{
			ChessMove[] moves;
			ChessBoard[] boards;
			double retValue;
			if (level == searchDepth)
			{				
				return board.GenerateBoardValue(forBoard.ColorToMove);
			}
			else
			{				
				board.GenerateSucessor(out moves, out boards);
				if (TerminalCondition(moves,board,level,out retValue))
					return retValue;
				if (VoluntaryDrawCondition(board))
					alpha = Math.Max(0.0,alpha);
				Array.Sort(moves,boards);
				nrGeneratedMoves += moves.Length;
				boardHistoryStack.Push(board);
				for(int i=0; i<boards.Length; ++i)
				{
					alpha = Math.Max(alpha,MinValue(boards[i],alpha,beta,level+1));
					if (alpha >= beta)
					{
						boardHistoryStack.Pop();
						return alpha;
						//return beta;
					}
				}
				boardHistoryStack.Pop();
				return alpha;
			}
		}
		private double MinValue(ChessBoard board, double alpha, double beta, int level)
		{
			ChessMove[] moves;
			ChessBoard[] boards;
			double retValue;
			if (level == searchDepth)
			{				
				return board.GenerateBoardValue(forBoard.ColorToMove);
			}
			else
			{				
				board.GenerateSucessor(out moves, out boards);
				if (TerminalCondition(moves,board,level,out retValue))
					return retValue;
				if (VoluntaryDrawCondition(board))
					beta = Math.Min(0.0,beta);
				Array.Sort(moves,boards);
				nrGeneratedMoves += moves.Length;
				boardHistoryStack.Push(board);
				for(int i=0; i<boards.Length; ++i)
				{
					beta = Math.Min(beta,MaxValue(boards[i],alpha,beta,level+1));
					if (beta <= alpha)
					{
						boardHistoryStack.Pop();
						return beta;
						//return alpha;
					}
				}
				boardHistoryStack.Pop();
				return beta;
			}
		}

		private void Shuffle(ChessMove[] moves, ChessBoard[] boards)
		{
			ChessMove[] movesCopy=(ChessMove[])moves.Clone();
			ChessBoard[] boardsCopy=(ChessBoard[])boards.Clone();
			Random rg=new Random();
			int randNr;
			for(int i=moves.Length; i>0; --i)
			{
				randNr = rg.Next(i);
				moves[i-1] = movesCopy[randNr];
				movesCopy[randNr] = movesCopy[i-1];
				boards[i-1] = boardsCopy[randNr];
				boardsCopy[randNr] = boardsCopy[i-1];
			}
		}

		private void ShuffleInClasses(ChessMove[] moves, ChessBoard[] boards)
		{
			int diffPrio=1+moves[0].Priority-moves[moves.Length-1].Priority;
			int index=0;
			ChessMove[][] movesCopy = new ChessMove[diffPrio][]; //(ChessMove[])moves.Clone();
            ChessBoard[][] boardsCopy = new ChessBoard[diffPrio][]; //(ChessBoard[])boards.Clone();
			ChessMove[] movesPart; //(ChessMove[])moves.Clone();
			ChessBoard[] boardsPart; //(ChessBoard[])boards.Clone();
			// Split moves and boards according to priority classes
			int startVal = moves[0].Priority;
			int StartIndex=0,NextIndex;
			do 
			{
				for(NextIndex=StartIndex; NextIndex < moves.Length && moves[NextIndex].Priority == startVal; NextIndex++)
				{
				}
				movesPart = new ChessMove[NextIndex-StartIndex];
				boardsPart = new ChessBoard[NextIndex-StartIndex];
				Array.Copy(moves,StartIndex,movesPart,0,NextIndex-StartIndex);
				Array.Copy(boards,StartIndex,boardsPart,0,NextIndex-StartIndex);
				movesCopy[index] = movesPart;
				boardsCopy[index] = boardsPart;
				index++;
				StartIndex = NextIndex;
				if (StartIndex < moves.Length)
					startVal = moves[StartIndex].Priority;
			} while (NextIndex < moves.Length);


			index = 0;
			for(int i=0; i<movesCopy.Length; ++i)
			{
				if (movesCopy[i] != null)
				{
					Shuffle(movesCopy[i],boardsCopy[i]);
					movesCopy[i].CopyTo(moves,index);
					boardsCopy[i].CopyTo(boards,index);
					index += movesCopy[i].Length;			
				}
			}

			// Shuffle the moves and boards

			// Reassemble moves and boards


			/*do
			{
			} while */

/*
			double startValue;
			int nrSame;
			if (boards.Length <= nrNodeToExpand)
			{
				selBoards = boards;
				selValues = values;
			}
			else
			{		
				selBoards = new ChessBoard[nrNodeToExpand];
				selValues = new double[nrNodeToExpand];
				startValue = values[0];
				nrSame = 1;
				for(int i=1; i<boards.Length; ++i)
				{
					if (Math.Abs(values[i]-startValue)>epsilon)
						break;
					nrSame++;
				}
				if (nrSame <= nrNodeToExpand)
				{
					Array.Copy(boards,selBoards,nrNodeToExpand);
					Array.Copy(values,selValues,nrNodeToExpand);
				}
				else
				{
					Random rg=new Random();
					int randNr;
					for(int i=nrNodeToExpand; i>0; --i)
					{
						randNr = rg.Next(i);
						selBoards[i-1] = boards[randNr];
						boards[randNr] = boards[i-1];
						selValues[i-1] = values[randNr];
						values[randNr] = values[i-1];
					}
				}
			}*/
		}
	}
}
