using System;
using System.Collections;
using System.Xml;

namespace ChessApplication
{
	/// <summary>
	/// Class with the behavior of the Bishop.
	/// </summary>
	public class Bishop : ChessPiece
	{
		/// <summary>
		/// Constructor of a Bishop.
		/// </summary>
		/// <param name="color"> The color or side of the Piece</param>
		/// <param name="board"> Reference to the ChessBoard this Piece is placed on</param>
        /// <param name="position"> The position on the board</param>
		/// <param name="relativeValue">The relative value of the Piece</param>
		/// <param name="movementBonus">The bonus amount for the Piece</param>/// 
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public Bishop(ChessPieceColor color, ChessBoard board, ChessPosition position, double relativeValue, double movementBonus) 
			: base(color,board,position,relativeValue,movementBonus) { }
		/// <summary>
		/// Deserialization constructor of a position on the Chessboard.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		public Bishop(XmlReader reader, ChessBoard board) 
			: base(reader,board) { }
		/// <summary>
		/// Get the total number of possible movement for this type of a piece.
		/// </summary>
		/// <returns> Returns possible movement</returns>
		public override int GetAbsoluteMovability()
		{
			return 13;
		}
		/// <summary>
		/// Generate all the possible legal moves for this piece at current state of the chessboard.		
		/// </summary>
		/// <param name="q"> The Queue to add the move to</param>
		/// <exception cref="ArgumentException">Thrown when the Queue q is null (not created)</exception>
		public override void AddLegalMoves(Queue q)
		{
			ChessPosition newPosition;
			ChessMoveType mtype;
			int i;
			if (q==null)
				throw new ArgumentException("q", "The queue is not created");
			//Diagonal right-up
			i = 1;
			while (ChessBoard.IsInsideBoard(position.Column+i,position.Row+i))
			{
				if (color == ChessPieceColor.White)
                    mtype = ChessMoveType.Forward;
				else
					mtype = ChessMoveType.Backward;
				newPosition = position.WithAddition(i,i);
				if (board.IsSqareOccupied(newPosition))
				{
					if (HasOppositeColor(newPosition))
						q.Enqueue(new ChessMove(position, newPosition, ChessMoveType.Capture));
					break;
				}
				else
				{
					q.Enqueue(new ChessMove(position, newPosition, mtype));
				}
				++i;
			}
			//Diagonal left-up
			i = 1;
			while (ChessBoard.IsInsideBoard(position.Column-i,position.Row+i))
			{
				if (color == ChessPieceColor.White)
					mtype = ChessMoveType.Forward;
				else
					mtype = ChessMoveType.Backward;
				newPosition = position.WithAddition(-i,i);
				if (board.IsSqareOccupied(newPosition))
				{
					if (HasOppositeColor(newPosition))
						q.Enqueue(new ChessMove(position, newPosition, ChessMoveType.Capture));
					break;
				}
				else
				{
					q.Enqueue(new ChessMove(position, newPosition, mtype));
				}
				++i;
			}
			//Diagonal left-down
			i = 1;
			while (ChessBoard.IsInsideBoard(position.Column-i,position.Row-i))
			{
				if (color == ChessPieceColor.White)
					mtype = ChessMoveType.Backward;					
				else
					mtype = ChessMoveType.Forward;
				newPosition = position.WithAddition(-i,-i);
				if (board.IsSqareOccupied(newPosition))
				{
					if (HasOppositeColor(newPosition))
						q.Enqueue(new ChessMove(position, newPosition, ChessMoveType.Capture));
					break;
				}
				else
				{
					q.Enqueue(new ChessMove(position, newPosition, mtype));
				}
				++i;
			}
			//Diagonal right-down
			i = 1;
			while (ChessBoard.IsInsideBoard(position.Column+i,position.Row-i))
			{
				if (color == ChessPieceColor.White)
					mtype = ChessMoveType.Backward;					
				else
					mtype = ChessMoveType.Forward;
				newPosition = position.WithAddition(i,-i);
				if (board.IsSqareOccupied(newPosition))
				{
					if (HasOppositeColor(newPosition))
						q.Enqueue(new ChessMove(position, newPosition, ChessMoveType.Capture));
					break;
				}
				else
				{
					q.Enqueue(new ChessMove(position, newPosition));
				}
				++i;
			}
		}
		/// <summary>
		/// Makes a good string of the state
		/// </summary>
		public override string ToString()
		{
			if (color == ChessPieceColor.White)
				return "Bishop<White,," + position + ">";
			else
				return "Bishop<Black,," + position + ">";
		}
	}
}
