using System;
using System.Collections;
using System.Xml;
using System.IO;

namespace ChessApplication 
{
	/// <summary>
	/// The error that made the move impossible.
	/// </summary>
	public enum ChessMoveErrorEnum {
		/// <summary> No error</summary>
		NoError,
		/// <summary> No piece at position tried to move</summary>
		NoPiece,
		/// <summary> Wrong side tried to move</summary>
		WrongSide,
		/// <summary> The move causes my king in Check</summary>
		PutMyKingInCheck,
		/// <summary> The game has already ended</summary>
		GameHasEnded,
		/// <summary> The move is not possible</summary>
		IllegalMove,
		/// <summary> The Draw Claim is not valid</summary>
		IllegalDrawClaim};
	/// <summary>
	/// The state of the game.
	/// </summary>
	public enum ChessGameState 
	{
		/// <summary> Ordinary situation</summary>
		Ordinary,
		/// <summary> Black king is checked</summary>
		BlackInCheck,
		/// <summary> White king is checked</summary>
		WhiteInCheck,
		/// <summary> Game has ended with draw</summary>
		Draw,
		/// <summary> Black has won the game</summary>
		BlackWon,
		/// <summary> White has won the game</summary>
		WhiteWon};

	/// <summary>
	/// The type of player.
	/// </summary>
	public enum ChessPlayerType
	{
		/// <summary> Ordinary human player</summary>
		Human,
		/// <summary> Computer player</summary>
		Computer,
		/// <summary> Remote human as client</summary>
		HumanIPNetworkClient,
		/// <summary> Remote human as server</summary>
		HumanIPNetworkServer,
		/// <summary> Game via email</summary>
		HumanEmail};

	/// <summary>
	/// Summary description for ChessBoard.
	/// </summary>
	public class ChessBoard : ICloneable, IEnumerable, IComparable 
	{
		private ChessPieceColor WhosTurn;
		private ChessGameState State;
		private ChessDrawType drawCause;
		private ChessPiece[,] board;
		private King BlackKingRef;
		private King WhiteKingRef;		
		private int nrMovePlayed;
		private int lastCaptureMoveNr;
		private int lastPawnMoveNr;
		private class ChessBoardEnumeratorArray : IEnumerator
		{
			IEnumerator en;
			public ChessBoardEnumeratorArray(ChessBoard chessboard)
			  { en = chessboard.board.GetEnumerator(); }
			public object Current 
			  { get { return en.Current; } }
			public bool MoveNext()
			{
				while (en.MoveNext())
				{
					if (en.Current != null)
						return true;
				}
				return false;
			}
			public void Reset()
			{
				en.Reset();
			}
		}
		private class ChessBoardEnumerator : IEnumerator
		{			
			private int i,j;
			private ChessBoard chessboard;
			private ChessPiece currentPiece;
			public ChessBoardEnumerator(ChessBoard chessboard)
			{				
				this.chessboard = chessboard;
				currentPiece = null;
				i = 0;
				j = 0;
			}
			public object Current
			{
				get 
				{					
					if (currentPiece == null)
						throw new InvalidOperationException();
					return currentPiece;
				}
			}
			public bool MoveNext()
			{
				currentPiece = null;
				while (i<8 && j<8)
				{
					if (chessboard.board[i,j] != null)
					{
						currentPiece = chessboard.board[i,j];
					}
					++j;
					if (j>=8)
					{
						++i;
						j=0;
					}
					if (currentPiece != null)
						return true;
				}
				return false;
			}
			public void Reset()
			{
				currentPiece = null;
				i = 0;
				j = 0;
			}
		}
		/// <summary>
		/// Gets the enumerator for the pieces on the board(enumerates even the "null" pieces).
		/// </summary>
		public IEnumerator GetEnumerator()
		{
			return new ChessBoardEnumerator(this);
		}
		/// <summary>
		/// Swap the color.
		/// </summary>
		/// <param name="cpc"> The color/side to swap</param>
		/// <returns> Returns the opposite color/side</returns>
		public static ChessPieceColor SwapColor(ChessPieceColor cpc)
		{
			if (cpc == ChessPieceColor.Black)
				return ChessPieceColor.White;
			else
				return ChessPieceColor.Black;
		}
		/// <summary>
		/// Constructor of a chessboard.
		/// </summary>
		public ChessBoard(double BishopValue, double KnightValue,
            double PawnValue, double QueenValue, double RookValue)
		{
			int column;		
		    double KingValue;
			// The king is worth more than every other Piece 
			// (considering a pawn can become a queen)			
			ChessPosition position;

			KingValue = 1+2*BishopValue+2*KnightValue+9*QueenValue+2*RookValue;
			WhosTurn = ChessPieceColor.White;
			State = ChessGameState.Ordinary;
			drawCause = ChessDrawType.NoDraw;
			nrMovePlayed = 0;
			lastCaptureMoveNr = 0;
		    lastPawnMoveNr = 0;
			board = new ChessPiece[8,8];

			for (int i = 0; i<8; i++)
				for (int j = 0; j<8; ++j)
					board[i,j] = null;
			position = new ChessPosition(1,8); SetPiece(position, new Rook(Rook.BlackColor,this,position,RookValue,1));
			position = new ChessPosition(8,8); SetPiece(position, new Rook(Rook.BlackColor,this,position,RookValue,1));
			position = new ChessPosition(2,8); SetPiece(position, new Knight(ChessPiece.BlackColor,this,position,KnightValue,1));
			position = new ChessPosition(7,8); SetPiece(position, new Knight(ChessPiece.BlackColor,this,position,KnightValue,1));
			position = new ChessPosition(3,8); SetPiece(position, new Bishop(ChessPiece.BlackColor,this,position,BishopValue,1));
			position = new ChessPosition(6,8); SetPiece(position, new Bishop(ChessPiece.BlackColor,this,position,BishopValue,1));
			position = new ChessPosition(4,8); SetPiece(position, new Queen(ChessPiece.BlackColor,this,position,QueenValue,1));
			position = new ChessPosition(5,8);
			BlackKingRef = new King(ChessPiece.BlackColor,this,position,KingValue,0);
			SetPiece(position, BlackKingRef);
			for (column = 1; column<=8; ++column)
			{
				position = new ChessPosition(column,7);
				SetPiece(position, new Pawn(ChessPiece.BlackColor,this,position,1,0.25));
			}
			position = new ChessPosition(1,1); SetPiece(position, new Rook(Rook.WhiteColor,this,position,RookValue,1));
			position = new ChessPosition(8,1); SetPiece(position, new Rook(Rook.WhiteColor,this,position,RookValue,1));
			position = new ChessPosition(2,1); SetPiece(position, new Knight(ChessPiece.WhiteColor,this,position,KnightValue,1));
			position = new ChessPosition(7,1); SetPiece(position, new Knight(ChessPiece.WhiteColor,this,position,KnightValue,1));
			position = new ChessPosition(3,1); SetPiece(position, new Bishop(ChessPiece.WhiteColor,this,position,BishopValue,1));
			position = new ChessPosition(6,1); SetPiece(position, new Bishop(ChessPiece.WhiteColor,this,position,BishopValue,1));
			position = new ChessPosition(4,1); SetPiece(position, new Queen(ChessPiece.WhiteColor,this,position,QueenValue,1));
			position = new ChessPosition(5,1);
			WhiteKingRef = new King(ChessPiece.WhiteColor,this,position,KingValue,0);
			SetPiece(position, WhiteKingRef);
			for (column = 1; column<=8; ++column)
			{
				position = new ChessPosition(column,2);
				SetPiece(position, new Pawn(ChessPiece.WhiteColor,this,position,1,0.25));
			}
		}
		/// <summary>
		/// Deserialization constructor of a Chessboard.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>
		public ChessBoard(XmlReader reader) 
		{
			ChessPiece cp;
			string PieceTypeStr;
			Type PieceType;
			board = new ChessPiece[8,8];
			for (int i = 0; i<8; i++)
				for (int j = 0; j<8; ++j)
					board[i,j] = null;	
			reader.ReadStartElement("ChessBoard");
			while (true)
				if (reader.IsStartElement("WhosTurn")) 
				{
					WhosTurn = (ChessPieceColor)XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("State")) 
				{
					State = (ChessGameState)XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("drawCause")) 
				{
					drawCause = (ChessDrawType)XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("nrMovePlayed")) 
				{
					nrMovePlayed = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("lastCaptureMoveNr")) 
				{
					lastCaptureMoveNr = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("lastPawnMoveNr")) 
				{
					lastPawnMoveNr = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.NodeType == XmlNodeType.Element)
				{
					PieceTypeStr = "ChessApplication" + "." + reader.Name;
					PieceType = Type.GetType(PieceTypeStr);
					cp = (ChessPiece)Activator.CreateInstance(PieceType,new Object[2]{reader,this});
					this.SetPiece(cp.Position,cp);
					if (cp is King)
					{
						if (cp.IsColorBlack())
							BlackKingRef = (King)cp;
						else
							WhiteKingRef = (King)cp;
					}
				} 
				else if ((reader.NodeType == XmlNodeType.EndElement) && (reader.Name == "ChessBoard")) 
				{
					reader.ReadEndElement();
					break;
				}
				else 
				{
					reader.Read();
					//throw new ApplicationException("Unknown node found");
				}
		}
		/// <summary>
		/// The color of the side who's turn it is to move.
		/// </summary>
		public ChessPieceColor ColorToMove
		{
			get
			{
				return WhosTurn;
			}
		}
		/// <summary>
		/// The state of the game.
		/// </summary>
		public ChessGameState GameState
		{
			get
			{
				return State;
			}
			set
			{
				State = value;
			}
		}
		/// <summary>
		/// The cause for draw.
		/// </summary>
		public ChessDrawType DrawCause
		{
			get
			{
				return drawCause;
			}
			set
			{
				drawCause = value;
			}
		}
		/// <summary>
		/// A reference to the black king.
		/// </summary>
		public King BlackKing
		{
			get
			{
				return BlackKingRef;
			}
		}
		/// <summary>
		/// A reference to the white king.
		/// </summary>
		public King WhiteKing
		{
			get
			{
				return WhiteKingRef;
			}
		}
		/// <summary>
		/// The number of moves currently played in this game.
		/// </summary>
		public int NumberOfMovePlayed
		{
			get
			{
				return nrMovePlayed;
			}
		}		
	
		/// <summary>
		/// Returns a piece on the chessboard.
		/// </summary>
		/// <param name="p"> The position of the Piece to get</param>
		/// <returns> Returns the chess piece</returns>
		public ChessPiece GetPiece(ChessPosition p)
		{
			return board[p.Column-1,p.Row-1];;
		}
		/// <summary>
		/// Returns a piece on the chessboard.
		/// </summary>
		/// <param name="column"> The column or line on the Chessboard in numercal format i.e. a=1, b=2, ...</param>
		/// <param name="row"> The row on the Chessboard</param>		
		/// <returns> Returns the chess piece</returns>
		public ChessPiece GetPiece(int column, int row)
		{
			return GetPiece(new ChessPosition(row,column));
		}
		/// <summary>
		/// Set a piece on the chessboard.
		/// </summary>
        /// <param name="p"> The position of the Piece to set</param>
		/// <param name="piece"> The chess piece to place on the square</param>
		public void SetPiece(ChessPosition p, ChessPiece piece)
		{
			board[p.Column-1,p.Row-1] = piece;			
		}

		private Queue GenerateAllMoves(ChessPieceColor color)
		{
			Queue q = new Queue();
			foreach(ChessPiece piece in this)
			{
				if (piece.Color == color)
				{
					piece.AddLegalMoves(q);					
				}
			}
			return q;
		}

		private void MoveToNotChecked(ChessPosition from, ChessPosition to)
		{
			nrMovePlayed++;
			GetPiece(from).MoveToNotChecked(to);
		}

		private ChessBoard[] Moves2Boards(Queue moves)
		{
			ChessBoard[] boards = new ChessBoard[moves.Count];
			ChessBoard   newBoard;
			int i;
			i = 0;
			foreach(ChessMove move in moves)
			{
				newBoard = (ChessBoard)Clone();
				newBoard.MoveToNotChecked(move.From,move.To);
				boards[i] = newBoard;
				++i;
			}
			newBoard = null;
			return boards;
		}

		/// <summary>
		/// Determines if a side has any valid moves.
		/// <param name="color"> The color/side to check if having any legal moves</param>
		/// </summary>
		public bool HasAnyLegalMove(ChessPieceColor color)
		{
			Queue q;
			ChessBoard[] boards;
			q = GenerateAllMoves(color);
			boards = Moves2Boards(q);
			foreach(ChessBoard cb in boards)
			{
				if (!cb.IsChecked(color))
				{
					q.Clear();
					boards = null;
					return true;
				}
			}
			q.Clear();
			boards = null;
			return false;
		}

		private double CalculateTotalValue(ChessPieceColor color)
		{
			double totalValue = 0;
			foreach(ChessPiece p in this)
			{
				if (p.Color == color)
					totalValue += p.GetRelativeValue();
			}
			return totalValue;
		}

		private double CalculateTotalValueMovmentBonus(ChessPieceColor color)
		{
			double totalValue = 0;
			foreach(ChessPiece p in this)
			{
				if (p.Color == color)
					totalValue += p.GetRelativeValue() 
						+ p.MovementBonus*p.GetActualMovability()/p.GetAbsoluteMovability();
			}
			return totalValue;
		}
		/// <summary>
		/// Compares the chess boards based on their piece values.
		/// </summary>
		/// 
		public int CompareTo(object obj)
		{
			if (obj is ChessBoard)
			{
				double myValue=GenerateBoardValue();
				double objValue=((ChessBoard)obj).GenerateBoardValue();
				if (myValue==objValue)
					return 0;
				if (myValue<objValue)
					return -1;
				else
					return 1;
			}
			else
				throw new ArgumentException("Must a ChessBoard to be able to compare","obj");
		}

		/// <summary>
		/// Calculate the value of the board for one side.
		/// </summary>
		/// <param name="forColor"> The color/side to evaluate the board value</param>
		/// <returns> Returns the value of the board</returns>
		public double GenerateBoardValue(ChessPieceColor forColor)
		{
			return CalculateTotalValue(forColor)
				- CalculateTotalValue(SwapColor(forColor));
		}

		/// <summary>
		/// Calculate the value of the board for the current player.
		/// </summary>
		/// <returns> Returns the value of the board</returns>
		public double GenerateBoardValue()
		{
			return GenerateBoardValue(WhosTurn);
		}

		/// <summary>
		/// Generates all the possible successor of the board,
		/// </summary>
		/// <param name="moves"> A array of the moves that is possible</param>
		/// <param name="boards"> A array of the resulting boars from the moves</param>
		public void GenerateSucessor(out ChessMove[] moves, out ChessBoard[] boards)
		{
			Queue q;
			ChessBoard newBoard;
			int moveNr, totalNrMoves;
			ChessMove[] generatedMoves;
			ChessBoard[] generatedBoards;
			q = GenerateAllMoves(WhosTurn);
			totalNrMoves = q.Count;
			generatedBoards = new ChessBoard[totalNrMoves];
			generatedMoves = new ChessMove[totalNrMoves];
			moveNr = 0;
			foreach(ChessMove move in q)
			{
				newBoard = (ChessBoard)Clone();
				newBoard.MoveToNotChecked(move.From,move.To);
				// Check that the move doesn't put us in check
				if (newBoard.IsChecked(newBoard.WhosTurn))
					continue;				
				newBoard.SwapTurnColor();
				generatedBoards[moveNr] = newBoard;
				generatedMoves[moveNr] = move;
				++moveNr;				
			}
			if (moveNr<totalNrMoves)
			{
				boards = new ChessBoard[moveNr];
				Array.Copy(generatedBoards,0,boards,0,moveNr);
				moves = new ChessMove[moveNr];
				Array.Copy(generatedMoves,0,moves,0,moveNr);
			}
			else
			{
				boards = generatedBoards;
				moves = generatedMoves;
			}
			newBoard = null;
			q.Clear();
			q = null;
		}

		/// <summary>
		/// Generates all the possible moves for one Piece.
		/// </summary>
		/// <param name="pos"> The Piece to check all posible new positions</param>
		/// <returns> Returns a Queue with all possible new positions</returns>
		public Queue GenerateNewPositionsForPiece(ChessPosition pos)
		{
			Queue all;
			Queue correct = new Queue();
			ChessBoard newBoard;
			if (GetPiece(pos)!=null)
			{
				all = GetPiece(pos).GenerateLegalMoves();
				foreach(ChessMove cm in all)
				{
					newBoard = (ChessBoard)Clone();
					newBoard.MoveToNotChecked(cm.From,cm.To);
					// Check that the move doesn't put us in check
					if (newBoard.IsChecked(newBoard.WhosTurn))
						continue;
					correct.Enqueue(cm.To);
				}
			}
			all = null;
            return correct;
		}

		/// <summary>
		/// Determines if a King is Checked.
		/// <param name="KingColor"> The color of the king to see if it's checked</param>
		/// </summary>
		public bool IsChecked(ChessPieceColor KingColor)
		{			
			if (KingColor == ChessPieceColor.Black)
				return BlackKing.IsChecked();
			else
				return WhiteKing.IsChecked();
		}

		private void SwapTurnColor()
		{
			if (WhosTurn == ChessPieceColor.Black)
				WhosTurn = ChessPieceColor.White;
			else
				WhosTurn = ChessPieceColor.Black;
		}

		/// <summary>
		/// Determines if a position is checked.
		/// </summary>
		/// <param name="testPosition"> The position to test</param>
		/// <param name="color"> The color/side to test for checking from</param>
		/// <returns> Returns true if position is checked</returns>		
		public bool IsPositionCheckedBy(ChessPosition testPosition, ChessPieceColor color)
		{
			Queue q;
			int movenr=0;
			foreach(ChessPiece piece in this)
			{
				if (piece.Color == color /*&& !(piece is King)*/)
				{
					if (piece is King)
					{
						q = new Queue();
						((King)piece).AddLegalMovesNoCastling(q);
						if (q.Contains(new ChessMove(piece.Position, testPosition)))
							return true;
					}
					else
					{
						q = piece.GenerateLegalMoves();
						if (q.Contains(new ChessMove(piece.Position, testPosition)))
							return true;
					}
						movenr++;
				}
			}
			return false;
		}


		private void SetGameState()
		{
			bool InCheck, AnyLegalMove;
			if (State == ChessGameState.Draw || State == ChessGameState.BlackWon
				|| State == ChessGameState.WhiteWon)
				return;
			InCheck = IsChecked(WhosTurn);
			AnyLegalMove = HasAnyLegalMove(WhosTurn);
			if (InCheck)
			{
				if (AnyLegalMove)
				{
					// Checked
					if (WhosTurn == ChessPieceColor.Black)
						State = ChessGameState.BlackInCheck;
					else
						State = ChessGameState.WhiteInCheck;
				}
				else
				{
					// Check-Mate
					if (WhosTurn == ChessPieceColor.Black)
						State = ChessGameState.WhiteWon;
					else
						State = ChessGameState.BlackWon;

				}
			}
			else
			{
				if (AnyLegalMove)
				{
					if (DrawToFewPiecesRule())
					{ // TO few pieces to ever be able to get check-mate
						State = ChessGameState.Draw;
						drawCause = ChessDrawType.RuleFewPieces;
					}
					else
					{ // Ordinary
						State = ChessGameState.Ordinary;
					}
				}
				else
				{
					// Draw
					State = ChessGameState.Draw;
					drawCause = ChessDrawType.UnableToMove;
				}
			}
		}

		/// <summary>
		/// Move a piece on the chessboard.
		/// </summary>
		/// <param name="from"> The position to move from</param>
		/// <param name="to"> The position to move to</param>
		/// <returns> Returns the error or lack of error</returns>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public ChessMoveErrorEnum MovePiece(ChessPosition from, ChessPosition to)
		{
			ChessPiece source = GetPiece(from);
			ChessBoard cb;
			if (State == ChessGameState.Draw || State == ChessGameState.BlackWon
				|| State == ChessGameState.WhiteWon)
				return ChessMoveErrorEnum.GameHasEnded;
			if (source == null)
				return ChessMoveErrorEnum.NoPiece;
			if (WhosTurn != source.Color)
				return ChessMoveErrorEnum.WrongSide;
			if (source.IsLegalMove(to))
			{
				cb = (ChessBoard)Clone();
				cb.GetPiece(from).MoveToNotChecked(to);
				if (cb.IsChecked(WhosTurn))
					return ChessMoveErrorEnum.PutMyKingInCheck;
				cb = null;
				nrMovePlayed++;
				source.MoveToNotChecked(to);				
				SwapTurnColor();
				SetGameState();		
			}
			else
			{
				return ChessMoveErrorEnum.IllegalMove;
			}
			return ChessMoveErrorEnum.NoError;
		}

		/// <summary>
		/// Move a piece on the chessboard.
		/// </summary>
		/// <param name="move"> The ChessMove to performe</param>
		/// <param name="boardHistoryStack"> Used to verify the validity of draw rules</param>
		/// <returns> Returns the error or lack of error</returns>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public ChessMoveErrorEnum MovePiece(ChessMove move, Stack boardHistoryStack)
		{
			if (State == ChessGameState.Draw || State == ChessGameState.BlackWon
				|| State == ChessGameState.WhiteWon)
				return ChessMoveErrorEnum.GameHasEnded;
			switch(move.DrawState)
			{
				case ChessDrawType.NoDraw:
					break;
				case ChessDrawType.UnableToMove:
					State = ChessGameState.Draw;
					drawCause = move.DrawState;
					return ChessMoveErrorEnum.NoError;
				case ChessDrawType.Rule50Move:
					if (this.Draw50MovesRule())
					{
						State = ChessGameState.Draw;
						drawCause = move.DrawState;
						return ChessMoveErrorEnum.NoError;
					}
					else
						return ChessMoveErrorEnum.IllegalDrawClaim;
				case ChessDrawType.RuleRepetition:
					if (this.DrawMoveRepetition(boardHistoryStack))
					{
						State = ChessGameState.Draw;
						drawCause = move.DrawState;
						return ChessMoveErrorEnum.NoError;
					}
					else
						return ChessMoveErrorEnum.IllegalDrawClaim;
				case ChessDrawType.RuleFewPieces:
					State = ChessGameState.Draw;
					drawCause = move.DrawState;
					return ChessMoveErrorEnum.NoError;
			}
			ChessPiece source = GetPiece(move.From);
			ChessBoard cb;
			if (source == null)
				return ChessMoveErrorEnum.NoPiece;
			if (WhosTurn != source.Color)
				return ChessMoveErrorEnum.WrongSide;
			if (source.IsLegalMove(move.To))
			{
				cb = (ChessBoard)Clone();
				cb.GetPiece(move.From).MoveToNotChecked(move.To);
				if (cb.IsChecked(WhosTurn))
					return ChessMoveErrorEnum.PutMyKingInCheck;
				cb = null;
				nrMovePlayed++;
				source.MoveToNotChecked(move.To);
				SwapTurnColor();
				SetGameState();		
			}
			else
			{
				return ChessMoveErrorEnum.IllegalMove;
			}
			return ChessMoveErrorEnum.NoError;
		}

		/// <summary>
		/// Determine if the a square is occupied.
		/// </summary>     
		/// <param name="position"> The position on the board</param>
		/// <returns> Returns true if the sqare is occupied</returns>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public bool IsSqareOccupied(ChessPosition position)
		{
			return GetPiece(position) != null;
		}
		/// <summary>
		/// Determine if the a square is black.
		/// </summary>
		/// <param name="column"> The column or line on the Chessboard in numercal format i.e. a=1, b=2, ...</param>
		/// <param name="row"> The row on the Chessboard</param>		
		/// <returns> Returns true if the sqare is black</returns>		
		static public bool IsSquareColorBlack(int column, int row)
		{
			return ((row+column)%2) == 0;
		}
		/// <summary>
		/// Determine if the a square is black.
		/// </summary>
		/// <param name="p"> The position on the chess board.</param>
		/// <returns> Returns true if the sqare is black</returns>		
		static public bool IsSquareColorBlack(ChessPosition p)
		{
			return IsSquareColorBlack(p.Column,p.Row);
		}
		/// <summary>
		/// Determine if the a square is white.
		/// </summary>
		/// <param name="column"> The column or line on the Chessboard in numercal format i.e. a=1, b=2, ...</param>
		/// <param name="row"> The row on the Chessboard</param>		
		/// <returns> Returns true if the sqare is white</returns>
		static public bool IsSquareColorWhite(int column, int row)
		{
			return ((row+column)%2) == 1;
		}
		/// <summary>
		/// Determine if the a square is inside the board.
		/// </summary>
		/// <param name="column"> The column or line on the Chessboard in numercal format i.e. a=1, b=2, ...</param>
		/// <param name="row"> The row on the Chessboard</param>		
		/// <returns> Returns true if the sqare inside the board</returns>
		static public bool IsInsideBoard(int column, int row)
		{
			return row>=1 && row<=8 && column>=1 && column<=8;
		}
		/// <summary>
		/// Determine if the a square is inside the board.
		/// </summary>
		/// <param name="position"> The position on the Chessboard</param>
		/// <returns> Returns true if the sqare inside the board</returns>
		static public bool IsInsideBoard(ChessPosition position)
		{
			return position.Row>=1 && position.Row<=8 && position.Column>=1 && position.Column<=8;
		}
		/// <summary>
		/// Gets the relative value of a queen.
		/// </summary>
		/// <returns> Returns the relative value of a queen</returns>
		public double GetQueenRelativeValue()
		{
			return 9;
		}
		/// <summary>
		/// Gets the bonus amount of a queen.
		/// </summary>
		/// <returns> Returns the bonus amount of a queen</returns>
		public double GetQueenMovementBonus()
		{
			return 0.3;
		}

		/// <summary>
		/// Generate a copy of the ChessBoard.
		/// </summary>
		/// <returns> Returns a copy of the ChessBoard</returns>
		public virtual object Clone()
		{
			ChessBoard newChessBoard = (ChessBoard)MemberwiseClone();
			newChessBoard.board = (ChessPiece[,])(board.Clone());
			for (int i = 0; i<8; i++)
				for (int j = 0; j<8; ++j)
				{
					if (newChessBoard.board[i,j] != null)
					{
						newChessBoard.board[i,j] = newChessBoard.board[i,j].Clone(newChessBoard);
						if (newChessBoard.board[i,j] is King)
						{
							if (newChessBoard.board[i,j].IsColorBlack())
								newChessBoard.BlackKingRef = (King)newChessBoard.board[i,j];
							else
								newChessBoard.WhiteKingRef = (King)newChessBoard.board[i,j];
						}
					}
				}
			return newChessBoard;
		}
		/// <summary>
		/// Xml saving function.
		/// </summary>
		/// <param name="writer"> The Xml writer</param>
		public virtual void SaveToXml(XmlWriter writer)
		{
			writer.WriteStartElement(this.GetType().Name);
			writer.WriteElementString("WhosTurn",XmlConvert.ToString((int)WhosTurn));
			writer.WriteElementString("State",XmlConvert.ToString((int)State));
			writer.WriteElementString("drawCause",XmlConvert.ToString((int)drawCause));
			writer.WriteElementString("nrMovePlayed",XmlConvert.ToString(nrMovePlayed));			
			writer.WriteElementString("lastCaptureMoveNr",XmlConvert.ToString(lastCaptureMoveNr));
			writer.WriteElementString("lastPawnMoveNr",XmlConvert.ToString(lastPawnMoveNr));
			foreach(ChessPiece cp in this)
			{
				cp.SaveToXml(writer);				
			}
			writer.WriteEndElement();
		}
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="obj"> Object to compair</param>
		public override bool Equals(object obj)
		{
			if (obj == null) return false;
			if (this.GetType() != obj.GetType()) 
				return false;
			if (WhosTurn != ((ChessBoard)obj).WhosTurn)
				return false;
			/*if (State != ((ChessBoard)obj).State)
				return false;
				*/
			for (int i = 0; i<8; i++)
				for (int j = 0; j<8; ++j)
					if (board[i,j] != ((ChessBoard)obj).board[i,j])
						return false;
			return true;
		}
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="objA"> Object to compair</param>
		/// <param name="objB"> Object to compair</param>
		public new static bool Equals(object objA, object objB)
		{
			if (objA == objB)
				return true;
			if (objA == null) return false;
			if (!(objA is ChessBoard))
				return false;
			return ((ChessBoard)objA).Equals(objB);
		}
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="A"> First piece to compair</param>
		/// <param name="B"> Second piece to compair</param> 
		/// <returns> Returns true if the chessboard are equal</returns>
		public static bool operator==(ChessBoard A, ChessBoard B)
		{
			return ChessBoard.Equals(A,B);
		}
		
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="A"> First piece to compair</param>
		/// <param name="B"> Second piece to compair</param> 
		/// <returns> Returns false if the chessboard are equal</returns>
		public static bool operator!=(ChessBoard A, ChessBoard B)
		{
			return !ChessBoard.Equals(A,B);
		}
		/// <summary>
		/// Creates a better hashcode.
		/// </summary>
		public override int GetHashCode()
		{
			return this.nrMovePlayed;
		}
		/// <summary>
		/// Set the last Pawn move to the current move.
		/// </summary>
		public void SetPawnMoveCurrent()
		{
			this.lastPawnMoveNr = nrMovePlayed;
		}
		/// <summary>
		/// Set the last capture move to the current move.
		/// </summary>
		public void SetCaptureMoveCurrent()
		{
			this.lastCaptureMoveNr = nrMovePlayed;
		}

		/// <summary>
		/// Determine if the "50 moves rules" are applicable that is:
		/// If there are have been 50 consecutive moves of white and of black without
		/// any piece taken 
		/// any pawn move 
		/// </summary>
		/// <returns> Returns true if the Draw rule is fullfilled</returns>
		public bool Draw50MovesRule()
		{
			if (nrMovePlayed >= 50)
			{
				if ((nrMovePlayed - lastPawnMoveNr) >= 50
					&& (nrMovePlayed - lastCaptureMoveNr) >= 50)
					return true;
				else
					return false;
			}
			else
				return false;
		}
		/// <summary>
		/// Determine if the 3 repetetiv states/moves rule are applicable.
		/// </summary>
		/// <param name="boards"> A array of the previous boards</param>
		/// <param name="realLength"> The realLengt if there are "junk" at the end</param>
		/// <returns> Returns true if the rule is fullfilled</returns>
		public bool DrawMoveRepetition(ChessBoard[] boards, int realLength)
		{
			if (boards.Length < 8 || realLength < 8 || boards.Length < realLength)
				return false;
			if (boards[realLength-4] == this && boards[realLength-8] == this)
				return true;
			else
				return false;
		}
		/// <summary>
		/// Determine if the 3 repetetiv states/moves rule are applicable.
		/// </summary>
		/// <param name="boards"> A array of the previous boards</param>
		/// <returns> Returns true if the rule is fullfilled</returns>
		public bool DrawMoveRepetition(ChessBoard[] boards)
		{
			return DrawMoveRepetition(boards,boards.Length);
		}
		/// <summary>
		/// Determine if the 3 repetetiv states/moves rule are applicable.
		/// </summary>
		/// <param name="boardHistory"> A stack of array of the previous boards</param>
		/// <returns> Returns true if the rule is fullfilled</returns>
		public bool DrawMoveRepetition(Stack boardHistory)
		{
			ChessBoard[] boards = new ChessBoard[8];
			int i=7;
			if (boardHistory.Count < 8)
				return false;
			foreach(ChessBoard b in boardHistory)
			{
				if (i>=0)
					boards[i] = b;
				else
					break;
				--i;
			}
			return DrawMoveRepetition(boards);
		}
		/// <summary>
		/// Determine if the "50 moves rules" are applicable that is:
		/// If there are have been 50 consecutive moves of white and of black without
		/// any piece taken 
		/// any pawn move 
		/// </summary>
		/// <returns> Returns true if the Draw rule is fullfilled</returns>
		public bool DrawToFewPiecesRule()
		{
			int nrBlackBishops=0, nrWhiteBishops=0;
			int nrBlackKnights=0, nrWhiteKnights=0;
			ChessPiece BlackBishop=null, WhiteBishop=null;
			int nrOthers=0;
			foreach(ChessPiece cp in this)
			{
				if (cp is Bishop) 
				{
					if (cp.IsColorBlack())
					{
						nrBlackBishops++;
						BlackBishop = cp;
					}
					else
					{
						nrWhiteBishops++;
						WhiteBishop = cp;
					}
				} 
				else if (cp is Knight)
				{
					if (cp.IsColorBlack())
						nrBlackKnights++;
					else
						nrWhiteKnights++;
				}
				else if (cp is King)
				{
				}
				else
				{
					nrOthers++;
				}
			}
			if (nrOthers>0)
				return false;
			// Only kings left, no chanse for Check-Mate
			if ((nrBlackBishops+nrWhiteBishops+nrBlackKnights+nrWhiteKnights)==0)
				return true;
			// king against king with only bishop or knight
			if ((nrBlackBishops+nrWhiteBishops+nrBlackKnights+nrWhiteKnights)==1)
				return true;
			if (nrBlackBishops==1 && nrWhiteBishops==1 &&
				IsSquareColorBlack(BlackBishop.Position)==IsSquareColorBlack(WhiteBishop.Position))
				return true;
			return false;
		}
		/// <summary>
		/// Constructor of a chessboard.
		/// </summary>
		public void ApplyPieceValues(double BishopValue, double KnightValue,
			double PawnValue, double QueenValue, double RookValue)
		{
			double KingValue;

			KingValue = 1+2*BishopValue+2*KnightValue+9*QueenValue+2*RookValue;

			foreach(ChessPiece cp in this)
			{
				if (cp is Bishop) 
					cp.RelativeValue = BishopValue;
				else if (cp is King) 
					cp.RelativeValue = KingValue;
				else if (cp is Knight) 
					cp.RelativeValue = KnightValue;
				else if (cp is Pawn) 
					cp.RelativeValue = PawnValue;
				else if (cp is Queen) 
					cp.RelativeValue = QueenValue;
				else if (cp is Rook) 
					cp.RelativeValue = RookValue;
			}
		}

		/// <summary>
		/// Give the number of pieces captured.
		/// </summary>
		/// <param name="BlackBishops">Number of Black Bishops captured</param>
		/// <param name="BlackKnights">Number of Black Knights captured</param>
		/// <param name="BlackPawns">Number of Black Pawns captured</param>
		/// <param name="BlackQueens">Number of Black Queens captured</param>
		/// <param name="BlackRooks">Number of Black Rooks captured</param>
		/// <param name="WhiteBishops">Number of White Bishops captured</param>
		/// <param name="WhiteKnights">Number of White Knights captured</param>
		/// <param name="WhitePawns">Number of White Pawns captured</param>
		/// <param name="WhiteQueens">Number of White Queens captured</param>
		/// <param name="WhiteRooks">Number of White Rooks captured</param>
		public void CapturedPieces(out int BlackBishops, out int BlackKnights,
			out int BlackPawns, out int BlackQueens, out int BlackRooks,
			out int WhiteBishops, out int WhiteKnights,
			out int WhitePawns, out int WhiteQueens, out int WhiteRooks)
		{
			BlackBishops = 2;
			BlackKnights = 2;
			BlackPawns = 8;
			BlackQueens = 1;
			BlackRooks = 2;
            WhiteBishops = 2;
			WhiteKnights = 2;
			WhitePawns = 8;
			WhiteQueens = 1;
			WhiteRooks = 2;
			foreach(ChessPiece cp in this)
			{
				if (cp is Bishop) 
				{
					if (cp.IsColorBlack())
						BlackBishops--;
					else
						WhiteBishops--;
				}
				else if (cp is King) 
				{
				}
				else if (cp is Knight) 
				{
					if (cp.IsColorBlack())
						BlackKnights--;
					else
						WhiteKnights--;
				}
				else if (cp is Pawn) 
				{
					if (cp.IsColorBlack())
						BlackPawns--;
					else
						WhitePawns--;
				}
				else if (cp is Queen) 
				{
					if (cp.IsColorBlack())
						BlackQueens--;
					else
						WhiteQueens--;
				}
				else if (cp is Rook) 
				{
					if (cp.IsColorBlack())
						BlackRooks--;
					else
						WhiteRooks--;
				}
			}
		}
	}
}