using System;
using System.Xml;
using System.Collections;

namespace ChessApplication
{
	/// <summary>
	/// The engine for generating moves for a chessboard.
	/// </summary>
	public abstract class ChessEngine
	{
		/*private class ChessEngineAsync : IAsyncResult
		{
			private Thread AsyncThread;
			private Mutex waiter;
			public ChessEngineAsync(ChessEngine ce)
			{
				AsyncThread = new Thread(new ThreadStart(ce.DecideBestMove()));
				waiter = new Mutex();
				AsyncThread.Start();
			}
			public bool IsCompleted
			{
				get { return AsyncThread.ThreadState != ThreadState.Stopped; }
			}
			public object AsyncState { get { return null; } }
			public WaitHandle AsyncWaitHandle { get { return waiter; } }
			public bool CompletedSynchronously { get { return false; } }
		}*/
		/// <summary>
		/// Delegate used to generate asyncronus calls to DecideBestMove
		/// </summary>
		protected delegate ChessMove DecideBestMoveDelegate();
		/// <summary>
		/// The ChessBoard to generated moves for.
		/// </summary>
		protected ChessBoard forBoard;
		/// <summary>
		/// The history of the chess game i.e. old ChessBoards.
		/// </summary>
		protected Stack boardHistoryStack;
		/// <summary>
		/// The depth of the moves to search for.
		/// </summary>
		protected int searchDepth;
		/// <summary>
		/// Counts the number of moves currently examined/generated
		/// </summary>
		protected int nrGeneratedMoves;
		/// <summary>
		/// Delegate used to generate asyncronus calls to DecideBestMove
		/// </summary>
		protected DecideBestMoveDelegate DecideBestMoveCaller;
		/// <summary>
		/// Constructor of a ChessEngine.
		/// </summary>
		/// <param name="board"> The board to generate moves for.</param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		protected ChessEngine(ChessBoard board, Stack boardHistoryStack)
		{
			this.forBoard = board;
			this.boardHistoryStack = boardHistoryStack;
			this.searchDepth = 4;
			this.DecideBestMoveCaller = null;
			nrGeneratedMoves = 0;
		}
		/// <summary>
		/// Constructor of a ChessEngine.
		/// </summary>
		/// <param name="board"> The board to generate moves for</param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		/// <param name="searchDepth"> The depth of the search</param>		
		protected ChessEngine(ChessBoard board, Stack boardHistoryStack, int searchDepth)
		{
			this.forBoard = board;
			this.boardHistoryStack = boardHistoryStack;
			this.searchDepth = searchDepth;
			this.DecideBestMoveCaller = null;
			nrGeneratedMoves = 0;
		}
		/// <summary>
		/// Deserialization constructor of a ChessEngine.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		public ChessEngine(XmlReader reader, ChessBoard board, Stack boardHistoryStack) 
		{
			this.forBoard = board;
			this.boardHistoryStack = boardHistoryStack;
			reader.ReadStartElement();
			while (true)
				if (reader.IsStartElement("searchDepth")) 
				{
					searchDepth = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if ((reader.NodeType == XmlNodeType.EndElement) && (reader.Name == this.GetType().Name)) 
				{
					reader.ReadEndElement();
					break;
				}
				else 
				{
					reader.Skip();
					//throw new ApplicationException("Unknown node found");
				}
		}
		/// <summary>
		/// The board to generate the moves for.
		/// </summary>
		public ChessBoard ForBoard
		{
			get { return forBoard; }
			set { forBoard = value; }
		}
		/// <summary>
		/// The history of the chess game i.e. old ChessBoards.
		/// </summary>
		public Stack BoardHistoryStack
		{
			get { return boardHistoryStack; }
			set { boardHistoryStack = value; }
		}
		/// <summary>
		/// The approximate number of nodes the have to be searched
		/// to decide the best move.
		/// </summary>
		/// <returns> Returns the number of nodes that will be searched</returns>
		public virtual int ApproxNrNodes()
		{
			return (int)Math.Pow(35,searchDepth);
		}

		/// <summary>
		/// The number currently generated moves possibilities for current move generation.
		/// </summary>
		public int NumberOfGeneratedMoves
		{
			get
			{
				return nrGeneratedMoves;
			}
		}
		/// <summary>
		/// How many moves them move generator should look ahead to determine the move.
		/// </summary>
		public int SearchDepth
		{
			get
			{
				return searchDepth;
			}
			set
			{
				searchDepth = value;
			}
		}
		/// <summary>
		/// Decides the best move and returns it.
		/// </summary>
		/// <returns> The best move using this engine</returns>
		public abstract ChessMove DecideBestMove();

		/// <summary>
		/// Call DecideBestMove asyncronusly.
		/// </summary>
		/// <returns> The Async result used to get the result</returns>
		public IAsyncResult BeginDecideBestMove()
		{
			if (DecideBestMoveCaller == null)
				throw new ArgumentNullException("DecideBestMoveCaller", "Not correctly set when used in the BeginDecideBestMove");
			return DecideBestMoveCaller.BeginInvoke(null,null);
		}

		/// <summary>
		/// End the asyncronus call to DecideBestMove.
		/// </summary>
		/// <param name="ar"> The Asyncronus result</param>
		/// <returns> The best move using this engine</returns>
		public ChessMove EndDecideBestMove(IAsyncResult ar)
		{
			return DecideBestMoveCaller.EndInvoke(ar);
		}

		/// <summary>
		/// Determine of a terminal condition has been reach i.e. Win,Loose,Draw
		/// </summary>
		/// <param name="moves"> Moves created as successors of board</param>
		/// <param name="board"> The current state</param>
		/// <param name="level"> The search level/depth</param>
		/// <param name="retValue"> The value to return of terminal condition has been reached</param>
		/// <returns> Returns true if terminal condition has been reached</returns>
		protected bool TerminalCondition(ChessMove[] moves, ChessBoard board, int level, out double retValue)
		{
			// Terminal condition, no possible move
			if (moves.Length == 0)
			{
				if (board.IsChecked(board.ColorToMove))
				{ 
					if (forBoard.ColorToMove == board.ColorToMove) 
						// I will be check-mate - give very bad value
						retValue = (searchDepth-level+1)*-forBoard.BlackKing.GetRelativeValue();
					else // Opponenet will be check-mate
						// Give very value for that, higher value the sooner we reach chess-mate
						retValue = (searchDepth-level+1)*forBoard.BlackKing.GetRelativeValue();
					if (board.ColorToMove == ChessPieceColor.Black)
						board.GameState = ChessGameState.WhiteWon;
					else 
						board.GameState = ChessGameState.BlackWon;
					return true;
				}
				else
				{
					retValue = 0;
					board.GameState = ChessGameState.Draw;
					board.DrawCause = ChessDrawType.UnableToMove;
					return true;
				}
			}
			else
			{
				retValue = 0;
				return false;
			}
/*		if (forBoard.ColorToMove == board.ColorToMove)
				{
					if (board.IsChecked(forBoard.ColorToMove)) // I will be check-mate
					{ 
						// Give very bad value for that
						retValue = (searchDepth-level+1)*-forBoard.BlackKing.GetRelativeValue();
						if (board.ColorToMove == ChessPieceColor.Black)
							board.GameState = ChessGameState.WhiteWon;
						else 
							board.GameState = ChessGameState.BlackWon;
						return true;
					}
					else // Since no legal move => Draw
					{
						retValue = 0;
						board.GameState = ChessGameState.Draw;
						board.DrawCause = ChessDrawType.UnableToMove;
						return true;
					}
				}
				else
				{
				}

					}
				}
				else
				{
					if (board.IsChecked(board.ColorToMove))
					{ // Opponenet will be check-mate
						// Give very value for that, higher value the sooner we reach chess-mate
						retValue = (searchDepth-level+1)*forBoard.BlackKing.GetRelativeValue();
						return true;
					}
					else // Since no legal move => Draw
					{
						retValue = 0;						
						return true;

					}

				}
			}
			else
			{
				retValue = 0;
				return false;
			}*/
		}
		/// <summary>
		/// Check if current side can claim draw.
		/// </summary>
		/// <param name="board"> THe board to make determination</param>
		/// <returns> Returns true if draw claim can be made</returns>
		protected bool VoluntaryDrawCondition(ChessBoard board)
		{
			if (board.Draw50MovesRule())
				return true;
			if (board.DrawMoveRepetition(boardHistoryStack))
				return true;
			return false;
		}
		/// <summary>
		/// Get the "real" board value considering the posibility of draw.
		/// </summary>
		/// <param name="board"> The chessboard to determine the value of</param>
		/// <param name="fValue"> The computed value</param>
		/// <returns> Returns the adjusted value</returns>
		protected double FilterForVoluntaryDrawCondition(ChessBoard board, double fValue)
		{
			if (VoluntaryDrawCondition(board))
			{
				if (forBoard.ColorToMove == board.ColorToMove)
					return Math.Max(0,fValue);
				else
					return Math.Min(0,fValue);
			}
			else
				return fValue;
		}
		/// <summary>
		/// Generate the values of a Array of ChessBoards.
		/// </summary>
		/// <param name="boards"> A Array of the boards to generate values for</param>
		/// <param name="forColor"> The color/side to evaluate the board value</param>
		/// <returns> Retuns a Array of the values of the boards</returns>
		protected double[] GenerateBoardValues(ChessBoard[] boards, ChessPieceColor forColor)
		{
			double[] values = new double[boards.Length];
			for(int i=0; i<boards.Length; ++i)
				values[i] = boards[i].GenerateBoardValue(forColor);
			return values;
		}

		/// <summary>
		/// Xml saving function.
		/// </summary>
		/// <param name="writer"> The Xml writer</param>
		public virtual void SaveToXml(XmlWriter writer)
		{
			writer.WriteStartElement(this.GetType().Name);
			writer.WriteElementString("searchDepth",XmlConvert.ToString(searchDepth));
			writer.WriteEndElement();
		}

		/// <summary>
		/// Generates the minimum value
		/// </summary>
		/// <param name="values"></param>
		/// <returns></returns>
		protected int MinIndex(double[] values)
		{
			if (values.Length <1)
				return 2*-1000;
			else
			{
				double min=values[0];
				int minIndex=0;
				for(int i=1; i<values.Length; ++i)
				{
					if (min<values[i])
					{
						min = values[i];
						minIndex = i;
					}
				}
				return minIndex;
			}
		}
	}
}
