using System;
using System.Xml;

namespace ChessApplication
{
	/// <summary>
	/// Manges the type of move, primarily used in AlphaBeta pruning.
	/// </summary>
	public enum ChessMoveType 
	{
		/// <summary> This will cause the piece to move backward on the chessboard</summary>
		Backward,
		/// <summary> Move with no particular preference</summary>
		Ordinary,
		/// <summary> This move will cause the piece to move forward on the Chess board</summary>
		Forward,
		/// <summary> This move will cause the piece to move forward on the Chess board and is particular valuable</summary>
		ForwardPlus,
		/// <summary> This move will cause the piece to move forward on the Chess board and is particular valuable</summary>
		ForwardPlus2,
		/// <summary> When this move causes a capture</summary>
		Capture
	};
	/// <summary>
	/// The Drawcause state of the game.
	/// </summary>
	public enum ChessDrawType
	{
		/// <summary> No draw</summary>
		NoDraw,
		/// <summary> Draw because the player is unable to move</summary>
		UnableToMove,
		/// <summary> The 50 moves rules</summary>
		Rule50Move,
		/// <summary> The repetition rules</summary>
		RuleRepetition,
		/// <summary> 
		/// (a)	king against king;
		/// (b)	king against king with only bishop or knight; 
		/// (c)	king and bishop against king and bishop, with both bishops on diagonals of the same colour.
		/// </summary>
		RuleFewPieces};
	/// <summary>
	/// Maintain a chess move.
	/// </summary>
	[Serializable()]
	public class ChessMove : ICloneable, IComparable
	{		
		private ChessPosition from, to;
		private ChessMoveType mtype;
		private ChessDrawType drawState;
		/// <summary>
		/// Constructor of a chess move.
		/// </summary>
		/// <param name="from"> The position to move from</param>
		/// <param name="to"> The position to move to</param>
		public ChessMove(ChessPosition from, ChessPosition to)
		{
			this.from = from;
			this.to = to;
			mtype = ChessMoveType.Ordinary;
			drawState = ChessDrawType.NoDraw;
		}
		/// <summary>
		/// Constructor of a chess move.
		/// </summary>
		/// <param name="from"> The position to move from</param>
		/// <param name="to"> The position to move to</param>
		/// <param name="mtype"> The type of the move</param>
		public ChessMove(ChessPosition from, ChessPosition to, ChessMoveType mtype)
		{
			this.from = from;
			this.to = to;
			this.mtype = mtype;
			drawState = ChessDrawType.NoDraw;
		}
		/// <summary>
		/// Constructor of a chess move.
		/// </summary>
		/// <param name="drawState"> The type of draw</param>
		private ChessMove(ChessDrawType drawState)
		{
			this.mtype = ChessMoveType.Ordinary;
			this.drawState = drawState;
		}
		/// <summary>
		/// Deserialization constructor of a position on the Chessboard.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		public ChessMove(XmlReader reader) 
		{
			bool hasReadTo=false;
			reader.ReadStartElement(this.GetType().Name);
			while (true)
				if (reader.IsStartElement("ChessPosition")) 
				{
					if (hasReadTo)
					{
						from = new ChessPosition(reader);
					}
					else
					{
						to = new ChessPosition(reader);
						hasReadTo = true;
					}										
				}
				else if ((reader.NodeType == XmlNodeType.EndElement) && (reader.Name == this.GetType().Name)) 
				{
					reader.ReadEndElement();
					break;
				}
				else 
				{
					reader.Read();
					//throw new ApplicationException("Unknown node found");
				}
		}
		/// <summary>
		/// Creates a ChessMove that is Claim of Draw based on 50 moves with 
		/// no capture or pawn move.
		/// </summary>
		/// <returns> Returns a Draw Claim move</returns>
		public static ChessMove ClaimDrawRule50Move()	
		{
			return new ChessMove(ChessDrawType.Rule50Move);
		}
		/// <summary>
		/// Creates a ChessMove that is Claim of Draw based on 3 repetetive states.
		/// </summary>
		/// <returns> Returns a Draw Claim move</returns>
		public static ChessMove ClaimDrawRuleRepetition()	
		{
			return new ChessMove(ChessDrawType.RuleRepetition);
		}
		/// <summary>
		/// Creates a ChessMove that is Claim of Draw based on no avaiable move.
		/// </summary>
		/// <returns> Returns a Draw Claim move</returns>
		public static ChessMove ClaimDrawUnableToMove()
		{
			return new ChessMove(ChessDrawType.UnableToMove);
		}
		/// <summary>
		/// Creates a ChessMove that there is to few pieces left i.e.
		/// (a)	king against king; 
		/// (b)	king against king with only bishop or knight; 
		/// (c)	king and bishop against king and bishop, with both bishops on diagonals of the same colour. 
		/// </summary>
		/// <returns> Returns a Draw Claim move</returns>
		public static ChessMove ClaimDrawRuleFewPieces()
		{
			return new ChessMove(ChessDrawType.RuleFewPieces);
		}
		/// <summary>
		/// Where the chess move is supposed to move from.
		/// </summary>
		public ChessPosition From
		{
			get 
			{
				return from;
			}
		}
		/// <summary>
		/// Where the chess move is supposed to move to.
		/// </summary>
		public ChessPosition To
		{
			get
			{
				return to;
			}
		}
		/// <summary>
		/// Get Draw state i.e. weather it is a claim to draw.
		/// </summary>
		public ChessDrawType DrawState
		{
			get
			{
				return drawState;
			}
		}

		/// <summary>
		/// The move type, used to determine preference to check best move
		/// </summary>
		public ChessMoveType MoveType
		{
			get 
			{
				return mtype;
			}
			set 
			{
				mtype = value;
			}
		}
		/// <summary>
		/// The priority of the, used to determine move to start examine.
		/// </summary>
		public int Priority
		{
			get 
			{
				return (int)mtype;
			}
		}
		/// <summary>
		/// Generate a copy of the move.
		/// </summary>
		/// <returns> Returns a copy of the move</returns>
		public virtual object Clone()
		{			
			return MemberwiseClone();
		}

		/// <summary>
		/// Makes a good string of the state
		/// </summary>
		public override string ToString()
		{			
			return "<" + from + "," + to + ">";
		}
		/// <summary>
		/// Creates a better hashcode.
		/// </summary>
		public override int GetHashCode()
		{
			return (from.Row << 24) | (from.Column << 16) | (to.Row << 8) | to.Column;
		}
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="obj"> Move to compair</param> 
		public override bool Equals(object obj)
		{
			if (obj == null) return false;
			if (this.GetType() != obj.GetType()) return false;
			if (from == ((ChessMove)obj).from && to == ((ChessMove)obj).to)
				return true;
			return false;  
		}
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="objA"> First move to compair</param>
		/// <param name="objB"> Second move to compair</param> 
		public static bool operator==(ChessMove objA, ChessMove objB)
		{
			return Equals(objA, objB);
		}
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="objA"> First move to compair</param>
		/// <param name="objB"> Second move to compair</param> 
		public static bool operator!=(ChessMove objA, ChessMove objB)
		{
			return !Equals(objA, objB);
		}

		/// <summary>
		/// Xml saving function.
		/// </summary>
		/// <param name="writer"> The Xml writer</param>
		public void SaveToXml(XmlWriter writer)
		{
			writer.WriteStartElement(this.GetType().Name);
			from.SaveToXml(writer);
			to.SaveToXml(writer);
			writer.WriteEndElement();
		}

		/// <summary>
		/// Compare ChessMoves based on movement typ/priority, when used with <code>System.Array.Sort()</code>
		/// it puts the most promising moves first
		/// </summary>
		/// <param name="o"> ChessMove to compair</param>
		/// <returns>
		/// Less than zero: This priority is higher than objs. 
		/// Zero: The priorities are equal
		/// Greater than zero: This priority is less than objs. 
		/// </returns>
		public int CompareTo(object o)
		{
			return ((ChessMove)o).mtype-mtype;
		}
	}
}
