using System;
using System.Collections;
using System.Xml;
using System.IO;

namespace ChessApplication
{
	/// <summary> Enum for maintaining the color/side on the chessboard.</summary>
	public enum ChessPieceColor 
	{
		/// <summary> Black color/side for the Piece.</summary>
		Black,
		/// <summary> White color/side for the Piece.</summary>
		White
	};

	/// <summary>
	/// The base class for the Chess pieces, implementning common methods and create placeholders for abstract methods.
	/// </summary>
	public abstract class ChessPiece : ICloneable
	{
		/// <summary> The Black color/side.</summary>
		public static ChessPieceColor BlackColor { get { return ChessPieceColor.Black; } }
		/// <summary> The White color/side.</summary>
		public static ChessPieceColor WhiteColor { get { return ChessPieceColor.White; } }
		/// <summary> The color or side of the Piece.</summary>
		protected ChessPieceColor color;
        /// <summary> Reference to the ChessBoard this Piece is placed on.</summary>
		protected ChessBoard board;
		/// <summary> The position on the board.</summary>
		protected ChessPosition position;
		/// <summary>The relative value of the Piece</summary>
		protected double relativeValue;
		/// <summary>The bonus amount for the Piece</summary>
		protected double movementBonus;
        /// <summary>Determine when piece moved last</summary>
		protected int lastMove;
		/// <summary>
		/// Constructor of a ChessPiece.
		/// </summary>
		/// <param name="color"> The color or side of the Piece</param>
		/// <param name="board"> Reference to the ChessBoard this Piece is placed on</param>
		/// <param name="position"> The position on the board</param>
		/// <param name="relativeValue">The relative value of the Piece</param>
		/// <param name="movementBonus">The bonus amount for the Piece</param>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public ChessPiece(ChessPieceColor color, ChessBoard board, ChessPosition position, double relativeValue, double movementBonus)
		{
			this.color = color;
			this.board = board;
			this.position = position;
			this.relativeValue = relativeValue;
			this.movementBonus = movementBonus;
			this.lastMove = 0;
		}
		/// <summary>
		/// Deserialization constructor of a position on the Chessboard.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		public ChessPiece(XmlReader reader, ChessBoard board) 
		{
			this.board = board;
			reader.ReadStartElement();
			while (true)
				if (reader.IsStartElement("color")) 
				{
					color = (ChessPieceColor)XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("relativeValue")) 
				{
					relativeValue = XmlConvert.ToDouble(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("movementBonus")) 
				{
					movementBonus = XmlConvert.ToDouble(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("nrMovePlayed")) 
				{
					lastMove = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("lastMove")) 
				{
					lastMove = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("ChessPosition"))
				{
					position = new ChessPosition(reader);
				}
				else if ((reader.NodeType == XmlNodeType.EndElement) && (reader.Name == this.GetType().Name)) 
				{
					reader.ReadEndElement();
					break;
				}
				else 
				{
					reader.Read();
					//throw new ApplicationException("Unknown node found");
				}
		}
		/// <summary> The color or side of the Piece.</summary>
		public ChessPieceColor Color
		{
			get { return color; }
		}
		/// <summary> The position on the board.</summary>
		public ChessPosition Position
		{
			get { return position; }
			set { position = value; }
		}
		/// <summary>The relative value of the Piece.</summary>
		public double RelativeValue
		{ 
			get { return relativeValue; } 
			set { relativeValue = value; }
		}
		
		/// <summary>The relative value of the Piece.</summary>
		public virtual double GetRelativeValue()
		{
			return relativeValue; 
		}
		/// <summary>The bonus amount for the Piece.</summary>
		public double MovementBonus
		{ get { return movementBonus; } }
		/// <summary> Weather the Piece has moved.</summary>
		public bool HasMoved
		{
			get { return lastMove>0; }
		}
		/// <summary> When the piece moves last.</summary>
		public int WhenLastMove
		{
			get { return lastMove; }
		}
		/// <summary>
		/// Generate a copy of the Piece, and in case of the sub classes that class.
		/// </summary>
		/// <returns> Returns a copy of the Piece</returns>
		public virtual object Clone()
		{
            // Return the actual type i.e in case the ChessPiece is a Pawn MemberwiseClone returns a Pawn
			return MemberwiseClone();
		}
		/// <summary>
		/// Generate a copy of the Piece, and in case of the sub classes that class.
		/// </summary>
		/// <returns> Returns a copy of the Piece</returns>
		public virtual ChessPiece Clone(ChessBoard cb)
		{
			// Return the actual type i.e in case the ChessPiece is a Pawn MemberwiseClone returns a Pawn
			ChessPiece cp = (ChessPiece)MemberwiseClone();
			cp.board = cb;
			return cp;
		}
		/// <summary>
		/// Move the Piece to another position, but only if it's a legal move.
		/// </summary>
        /// <param name="newPosition"> The position to move to</param>
        /// <returns> Returns true if the move is legal and false if not</returns>
		public virtual bool MoveTo(ChessPosition newPosition)
		{
			if (IsLegalMove(newPosition))
			{
				MoveToNotChecked(newPosition);
				return true;
			}
			else
				return false;
		}
		/// <summary>
		/// Move the Piece to another position without verify that it is a correct move.
		/// </summary>
        /// <param name="newPosition"> The position to move to</param>
		public virtual void MoveToNotChecked(ChessPosition newPosition)
		{
			if (board.IsSqareOccupied(newPosition))
				board.SetCaptureMoveCurrent();
			board.SetPiece(newPosition, this);
			board.SetPiece(position, null);
			position = newPosition;
		    lastMove = board.NumberOfMovePlayed;
		}
		/// <summary>
		/// Determine if color/side is Black.
		/// </summary>
		/// <returns> Returns true if the color/side is Black</returns>
		public bool IsColorBlack()
		{
			return color==ChessPieceColor.Black;
		}
		/// <summary>
		/// Determine if color/side is White.
		/// </summary>
		/// <returns> Returns true if the color/side is White</returns>
		public bool IsColorWhite()
		{
			return color==ChessPieceColor.White;
		}
		/// <summary>
		/// Get the total number of possible movement for this type of a piece.
		/// </summary>
		/// <returns> Returns possible movement</returns>
		public abstract int GetAbsoluteMovability();
		/// <summary>
		/// Get the actual number of possible movement for this type of a piece at current game state.
		/// </summary>
		/// <returns> Returns number possible movement</returns>
		public virtual int GetActualMovability()
		{
			return GenerateLegalMoves().Count;
		}
		/// <summary>
		/// Get the color/side that is opposite to this piece.
		/// </summary>
		/// <returns> Returns the opposite color/side</returns>
		public ChessPieceColor GetOppositeColor()
		{
			if (IsColorWhite())
				return ChessPieceColor.Black;				
			else
				return ChessPieceColor.White;
				
		}
		/// <summary>
		/// Determine if the other Piece has the opposite color.
		/// </summary>
        /// <param name="otherPosition"> The position check</param> 
		/// <returns> Returns true if opposite color</returns>
		/// <exception cref="ArgumentException"> Thrown if no piece at position</exception>
        public bool HasOppositeColor(ChessPosition otherPosition)
		{
			if (!board.IsSqareOccupied(otherPosition))
				throw new ArgumentException("otherPosition","The position doesn't contain a piece, can't determine color/side of the piece");
			return board.GetPiece(otherPosition).Color == GetOppositeColor();
		}
		/// <summary>
		/// Generate all the possible legal moves for this piece at current state of the chessboard.		
		/// </summary>
		/// <param name="q"> The Queue to add the move to</param>
		public abstract void AddLegalMoves(Queue q);
		/// <summary>
		/// Generate all the possible legal moves for this piece at current state of the chessboard.		
		/// </summary>
		/// <returns> Returns a Queue of with the legal moves</returns>
		public virtual Queue GenerateLegalMoves()
		{	
			Queue q = new Queue();
			AddLegalMoves(q);
			return q;
		}
		/// <summary>
		/// Determine if a move i legal.		
		/// </summary>
		/// <param name="newPosition"> The position to move to</param>
		/// <returns> Returns true if the move is legal</returns>
		public virtual bool IsLegalMove(ChessPosition newPosition)
		{
			Queue q = GenerateLegalMoves();
			if (q == null || q.Count == 0)
				return false;
			if (q.Contains(new ChessMove(position,newPosition)))
				return true;
			return false;
		}

		/// <summary>
		/// Xml saving function.
		/// </summary>
		/// <param name="writer"> The Xml writer</param>
		public virtual void SaveToXml(XmlWriter writer)
		{
			writer.WriteStartElement(this.GetType().Name);
			writer.WriteElementString("color",XmlConvert.ToString((int)color));
			position.SaveToXml(writer);			
			writer.WriteElementString("relativeValue",XmlConvert.ToString(relativeValue));
			writer.WriteElementString("movementBonus",XmlConvert.ToString(movementBonus));
			writer.WriteElementString("lastMove",XmlConvert.ToString(lastMove));			
			writer.WriteEndElement();
		}

		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="obj"> Object to compair</param>
		/// <returns> Returns true if the chesspieces are equal</returns>
		public override bool Equals(object obj)
		{
			if (obj == null) return false;
			if (this.GetType() != obj.GetType()) 
				return false;
			if (color != ((ChessPiece)obj).color)
				return false;
			if (position != ((ChessPiece)obj).position)
				return false;
			if (Math.Abs(relativeValue-((ChessPiece)obj).relativeValue)>0.001)
				return false;
			if (Math.Abs(movementBonus-((ChessPiece)obj).movementBonus)>0.001)
				return false;
			/*if (hasMoved != ((ChessPiece)obj).hasMoved)
				return false;*/
			return true;
		}
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="objA"> Object to compair</param>
		/// <param name="objB"> Object to compair</param>
		/// <returns> Returns true if the chesspieces are equal</returns>
		public new static bool Equals(object objA, object objB)
		{
			if (objA == objB)
				return true;
			if (objA == null) return false;
			if (!(objA is ChessPiece))
				return false;
			return ((ChessPiece)objA).Equals(objB);
		}
		
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="A"> First piece to compair</param>
		/// <param name="B"> Second piece to compair</param> 
		/// <returns> Returns true if the chesspieces are equal</returns>
		public static bool operator==(ChessPiece A, ChessPiece B)
		{
			return ChessPiece.Equals(A,B);
		}
		
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="A"> First piece to compair</param>
		/// <param name="B"> Second piece to compair</param> 
		/// <returns> Returns false if the chesspieces are equal</returns>
		public static bool operator!=(ChessPiece A, ChessPiece B)
		{
			return !ChessPiece.Equals(A,B);
		}
		/// <summary>
		/// Creates a better hashcode.
		/// </summary>
		/// /// <returns> Returns the hashcode based on the position</returns>
		public override int GetHashCode()
		{
			return position.GetHashCode();
		}
	}
}
