using System;
using System.Xml;
using System.IO;

namespace ChessApplication
{
	/// <summary>
	/// Maintain a position on the Chessboard.
	/// </summary>
	[Serializable()]
	public struct ChessPosition /*: ICloneable*/
	{
		/// <summary> Column A.</summary>
		public static int Column_A { get { return 1; } }
		/// <summary> Column B.</summary>
		public static int Column_B { get { return 2; } }
		/// <summary> Column C.</summary>
		public static int Column_C { get { return 3; } }
		/// <summary> Column D.</summary>
		public static int Column_D { get { return 4; } }
		/// <summary> Column E.</summary>
		public static int Column_E { get { return 5; } }
		/// <summary> Column F.</summary>
		public static int Column_F { get { return 6; } }
		/// <summary> Column G.</summary>
		public static int Column_G { get { return 7; } }
		/// <summary> Column H.</summary>
		public static int Column_H { get { return 8; } }
		/// <summary>
		/// The column or line on the Chessboard in numercal format i.e. a=1, b=2, ....
		/// </summary>
		public int Column;
		/// <summary>
		/// The row on the Chessboard.
		/// </summary>
		public int Row;
		/// <summary>
		/// Constructor of a position on the Chessboard.
		/// </summary>
		/// <param name="column"> The column or line on the Chessboard in numercal format i.e. a=1, b=2, ...</param>
		/// <param name="row"> The row on the Chessboard</param>		
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public ChessPosition(int column, int row)
		{
			if (row<1 || row>8)
				throw new ArgumentOutOfRangeException("row", "The row has to be in the range [1,8] to be a correct Chessboard position");
			if (column<1 || column>8)
				throw new ArgumentOutOfRangeException("column", "The column has to be in the range [1,8] to be a correct Chessboard position");
			this.Row = row;
			this.Column = column;
		}		
		/// <summary>
		/// Deserialization constructor of a position on the Chessboard.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		public ChessPosition(XmlReader reader) 
		{
			Column = 1;
			Row = 1;
			reader.ReadStartElement(this.GetType().Name);
			while (true)
				if (reader.IsStartElement("Column")) 
				{
					Column = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("Row")) 
				{
					Row = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
			    else if ((reader.NodeType == XmlNodeType.EndElement) && (reader.Name == this.GetType().Name)) 
				{
					reader.ReadEndElement();
					break;
				}
				else 
				{
					reader.Read();
					//throw new ApplicationException("Unknown node found");
				}
		}
		/// <summary>
		/// Generate a copy of the position.
		/// </summary>
		/// <param name="columnAdd"> The addition to the column</param>
		/// <param name="rowAdd"> The addition to the row</param>		
		public ChessPosition WithAddition(int columnAdd, int rowAdd)
		{
			ChessPosition p;
            p.Row = Row+rowAdd;
			p.Column = Column+columnAdd;
			return p;
		}
		/*
		/// <summary>
		/// Generate a copy of the position.
		/// </summary>
		/// <returns> Returns a copy of the position</returns>
		public object Clone()
		{			
			return MemberwiseClone();
		}
		*/
/*		/// <summary>
		/// Get the row on the Chessboard.
		/// </summary>
		/// <returns> Returns the row. </returns>
		public int GetRow() { return Row; }
		/// <summary>
		/// Get the column or line on the Chessboard in numercal format instead if letter format i.e. 1=a, 2=b, ...
		/// </summary>
		/// /// <returns> Returns the column/line. </returns>
		public int GetColumn() { return Column; }
		*/
		/// <summary>
		/// Makes a good string of the state
		/// </summary>
		public override string ToString()
		{
            char colLetter=(char)((int)'a'+Column-1);
			return "(" + colLetter + "," + Row + ")";
		}
		/// <summary>
		/// Creates a better hashcode.
		/// </summary>
		public override int GetHashCode()
		{
			return (Row << 16) | Column;
		}

		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="obj"> Object to compair</param>
		/// <returns> Returns true if the ChessPositions are equal</returns>
		public override bool Equals(object obj)
		{
			if (obj == null) return false;
			if (this.GetType() != obj.GetType()) return false;
			if (Row == ((ChessPosition)obj).Row && Column == ((ChessPosition)obj).Column)
				return true;
			return false;  
		}
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="objA"> Object to compair</param>
		/// <param name="objB"> Object to compair</param>
		/// <returns> Returns true if the ChessPositions are equal</returns>
		public new static bool Equals(object objA, object objB)
		{
			if (objA == objB)
				return true;
			if (objA == null) return false;
			if (!(objA is ChessPosition))
				return false;
			return ((ChessPosition)objA).Equals(objB);
		}
		
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="posA"> First position to compair</param>
		/// <param name="posB"> Second position to compair</param>
		/// <returns> Returns true if the ChessPositions are equal</returns>
		public static bool operator==(ChessPosition posA, ChessPosition posB)
		{
			return ChessPosition.Equals(posA,posB);
			/*if (posA == null || posB == null)
				return false;
			else
				return (posA.Column == posB.Column) && (posA.Row == posB.Row);*/
		}
		
		/// <summary>
		/// Equality should be based on State not references.
		/// </summary>
		/// <param name="posA"> First position to compair</param>
		/// <param name="posB"> Second position to compair</param> 
		/// <returns> Returns true if the ChessPositions are NOT equal</returns>
		public static bool operator!=(ChessPosition posA, ChessPosition posB)
		{
			return !ChessPosition.Equals(posA,posB);
			//return !(posA == posB);
		}

		/// <summary>
		/// Xml saving function.
		/// </summary>
		/// <param name="writer"> The Xml writer</param>
		public void SaveToXml(XmlWriter writer)
		{
			writer.WriteStartElement(this.GetType().Name);
			writer.WriteElementString("Column",XmlConvert.ToString(Column));
			writer.WriteElementString("Row",XmlConvert.ToString(Row));
			writer.WriteEndElement();
		}
		
	}
}
