using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace ChessApplication
{
	/// <summary>
	/// A delegate type for hooking up ImageGrid events.
	/// </summary>
	/// <param name="sender"> The sending object (will be a ImageGrid)</param>
	/// <param name="p"> The Point on the square</param>
	public delegate void ImageGridEventHandler(object sender, Point p);
	/// <summary>
	/// A delegate type for hooking up ImageGrid events.
	/// </summary>
	/// <param name="sender"> The sending object (will be a ImageGrid)</param>
	/// <param name="from"> The start point</param>
	/// <param name="to"> The end point</param>
	public delegate void ImageGridSelectionChangeEventHandler(object sender, Point from, Point to);
	/// <summary>
	/// Maintans a Grid of images.
	/// </summary>
	public class ImageGrid : System.Windows.Forms.UserControl
	{
		/// <summary>
		/// The event fired if there is a click in one square.
		/// </summary>
		public event ImageGridEventHandler SquareClick;
		/// <summary>
		/// The event fired when a square is unselected.
		/// </summary>
		public event ImageGridEventHandler UnSelected;
		/// <summary>
		/// The event fired when a square is selected.
		/// </summary>
		public event ImageGridEventHandler Selected;
		/// <summary>
		/// The event fired when the mouse is held down an moved to another square.
		/// </summary>
		public event ImageGridSelectionChangeEventHandler Dragged;
		/// <summary>
		/// The event fired when a square is selected instead of another square.
		/// </summary>
		public event ImageGridSelectionChangeEventHandler SelectionChanged;
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;


		private Size NoImagesSize;
		private Point Selection;
		private Point MouseDownPoint;
		private Point DragStart;

		private Image [,] Images;
		private Brush [,] Brushes;
		private bool [,] IsHighlighted;

		private Brush LightBrush;
		private Brush DarkBrush;

		private Color SelColor;
		private Color HighlighColor;

		private float SelWidth;

        private bool HasLabels;
		private bool LabelAtTop;
		private bool LabelAtLeft;
		private string[] XLabels;
		private string[] YLabels;
		private float RelLabelSize;

		private Brush LabelBrush;
		private Brush LabelBackgroundBrush;
		//private string LabelFontName;

		/// <summary> 
		/// Creates a ImageGrid.
		/// </summary>
		public ImageGrid()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();            
			// TODO: Add any initialization after the InitForm call
			NoImagesSize = new Size(8,8);
			Images = new Image[NoImagesSize.Width, NoImagesSize.Height];
			Brushes = new Brush[NoImagesSize.Width, NoImagesSize.Height];
			IsHighlighted = new bool[NoImagesSize.Width, NoImagesSize.Height];
            DarkBrush = new SolidBrush(Color.Red);
		    LightBrush = new SolidBrush(Color.White);
			Selection = new Point(-1,-1);
			DragStart = new Point(-1,-1);
			MouseDownPoint = new Point(-1,-1);
			SelColor = Color.Purple;
			SelWidth = 0.1F;
			HighlighColor = Color.Green;
			HasLabels = true;
			LabelAtTop = false;
			LabelAtLeft = true;
			XLabels = new string[8] {"a","b","c","d","e","f","g","h"};
			YLabels = new string[8] {"8","7","6","5","4","3","2","1"};
			RelLabelSize = 0.5F;
		    LabelBrush = new SolidBrush(Color.Black);
			LabelBackgroundBrush = new SolidBrush(Color.Beige);
		    //LabelFontName = "Ariel";

			SetStyle(ControlStyles.UserPaint, true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer, true);
			
		}
		/// <summary> 
		/// The size of the grid in number of images.
		/// </summary>
		public Size GridSize 
		{
			get 
			{
				return NoImagesSize;
			}
			set
			{
				NoImagesSize = value;
				Images = new Image[NoImagesSize.Width, NoImagesSize.Height];
				Brushes = new Brush[NoImagesSize.Width, NoImagesSize.Height];
				IsHighlighted = new bool[NoImagesSize.Width, NoImagesSize.Height];
				if (NoImagesSize.Width == 8 && NoImagesSize.Height == 8)
				{
					XLabels = new string[8] {"a","b","c","d","e","f","g","h"};
					YLabels = new string[8] {"8","7","6","5","4","3","2","1"};
				}
				else
				{
					XLabels = new string[NoImagesSize.Width];
					YLabels = new string[NoImagesSize.Height];
					for(int i=0; i<NoImagesSize.Width; i++)
						XLabels[i] = "";
					for(int i=0; i<NoImagesSize.Height; i++)
						YLabels[i] = "";
				}
			}
		}

		/// <summary>
		/// Set the light brush used as backgound for images at even coordinates sum
		/// </summary>
		public Brush LightBackBrush
		{
			get	{ return LightBrush;}
			set { LightBrush = value;}
		}
		/// <summary>
		/// Set the dark brush used as backgound for images at uneven coordinates sum
		/// </summary>
		public Brush DarkBackBrush
		{
			get	{ return DarkBrush;}
			set { DarkBrush = value;}
		}

		/// <summary>
		/// Set the brush used as backgound.
		/// </summary>
		public Brush BackBrush
		{
			get	{ return LabelBackgroundBrush;}
			set { LabelBackgroundBrush = value;}
		}
		/// <summary>
		/// Set the brush used as foregound.
		/// </summary>
		public Brush ForeBrush
		{
			get	{ return LabelBrush;}
			set { LabelBrush = value;}
		}

		/// <summary> 
		/// The Selected Image.
		/// To deselect set x or y to less than 0
		/// </summary>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x or y is not within the ImageGrid.</exception>
		public Point SelectedImage
		{
			get
			{
				return Selection;
			}
			set
			{
				if (value.X>=NoImagesSize.Width)
					throw new ArgumentOutOfRangeException("value.X", "The x coordinate is \"" + value.X + "\" but has to be in <" + NoImagesSize.Width + " to be within the ImageGrid");
				if (value.Y>=NoImagesSize.Height)
					throw new ArgumentOutOfRangeException("value.Y", "The y coordinate is \"" + value.Y + "\" but has to be in <" + NoImagesSize.Height + " to be within the ImageGrid");
				if (Selection != value)
				{
					OnUnSelect();
					Selection = value;
					OnSelect();
				}
			}
		}
		/// <summary> 
		/// The Color of the rectangle indicating the Selected Square.
		/// </summary>
		public Color SelectionColor
		{
			get
			{
				return SelColor;
			}
			set
			{
				SelColor = value;
			}
		}
		/// <summary> 
		/// The Color of the rectangle indicating the highlighted Square.
		/// </summary>
		public Color HighlightingColor
		{
			get
			{
				return HighlighColor;
			}
			set
			{
				HighlighColor = value;
			}
		}
		/// <summary> 
		/// The Line width of the rectangle indicating the Selected Square.
		/// </summary>
		public float SelectionLineWidth
		{
			get
			{
				return SelWidth;
			}
			set
			{
				SelWidth = value;
			}

		}
		/// <summary> 
		/// Determines if there should be labels for the rows and columns in the grid.
		/// </summary>
		public bool HasGridLabels
		{
			get
			{
				return HasLabels;
			}
			set
			{
				HasLabels = value;
			}
		}
		/// <summary> 
		/// Sets the x axis labels to be to the top of the grid.
		/// </summary>
		public void SetXLabelsToTop()
		{
			LabelAtTop = true;
		}
		/// <summary> 
		/// Sets the x axis labels to be to the bottom of the grid.
		/// </summary>
		public void SetXLabelsToBottom()
		{
			LabelAtTop = false;
		}
		/// <summary> 
		/// Determine where the x axis labels are.
		/// </summary>
		/// <returns> Returns true if the x axis labels are at the top of the grid</returns>
		public bool IsXLabelsAtTop()
		{
			return LabelAtTop;
		}
		/// <summary> 
		/// Sets the y axis labels to be to the left of the grid.
		/// </summary>
		public void SetYLabelsToLeft()
		{
			LabelAtLeft = true;
		}
		/// <summary> 
		/// Sets the y axis labels to be to the right of the grid.
		/// </summary>
		public void SetYLabelsToRight()
		{
			LabelAtLeft = false;
		}
		/// <summary> 
		/// Determine where the y axis labels are.
		/// </summary>
		/// <returns> Returns true if the y axis labels are to left of the grid</returns>
		public bool IsYLabelsAtLeft()
		{
			return LabelAtLeft;
		}
		/// <summary>
		/// Set a label of the x axis.
		/// </summary>
		/// <param name="xCoordinate"> The x coordinate of the label</param>
		/// <param name="xLabel"> The label to set</param>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x is not within the ImageGrid.</exception>
		public void SetXLabel(int xCoordinate, string xLabel)
		{
			if (xCoordinate<0 || xCoordinate>=NoImagesSize.Width)
				throw new ArgumentOutOfRangeException("xCoordinate", "The x coordinate is \"" + xCoordinate + "\" but has to be in the range [0," + NoImagesSize.Width + "] to be within the ImageGrid");
			XLabels[xCoordinate] = xLabel;
		}
		/// <summary>
		/// Get a label of the x axis.
		/// </summary>
		/// <param name="xCoordinate"> The x coordinate of the label</param>
		/// <returns> Returns the label of the specified x-coordinate</returns>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x is not within the ImageGrid.</exception>
		public string GetXLabel(int xCoordinate)
		{
			if (xCoordinate<0 || xCoordinate>=NoImagesSize.Width)
				throw new ArgumentOutOfRangeException("xCoordinate", "The x coordinate is \"" + xCoordinate + "\" but has to be in the range [0," + NoImagesSize.Width + "] to be within the ImageGrid");
			return XLabels[xCoordinate];
		}
		/// <summary>
		/// Set a label of the y axis.
		/// </summary>
		/// <param name="yCoordinate"> The y coordinate of the label</param>
		/// <param name="yLabel"> The label to set</param>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when y is not within the ImageGrid.</exception>
		public void SetYLabel(int yCoordinate, string yLabel)
		{
			if (yCoordinate<0 || yCoordinate>=NoImagesSize.Height)
				throw new ArgumentOutOfRangeException("yCoordinate", "The y coordinate is \"" + yCoordinate + "\" but has to be in the range [0," + NoImagesSize.Height + "] to be within the ImageGrid");
			YLabels[yCoordinate] = yLabel;
		}
		/// <summary>
		/// Get a label of the y axis.
		/// </summary>
		/// <param name="yCoordinate"> The y coordinate of the label</param>
		/// <returns> Returns the label of the specified y-coordinate</returns>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when y is not within the ImageGrid.</exception>
		public string GetYLabel(int yCoordinate)
		{
			if (yCoordinate<0 || yCoordinate>=NoImagesSize.Height)
				throw new ArgumentOutOfRangeException("yCoordinate", "The x coordinate is \"" + yCoordinate + "\" but has to be in the range [0," + NoImagesSize.Height + "] to be within the ImageGrid");
			return YLabels[yCoordinate];
		}
		/// <summary>
		/// Set the Brush used as background at a specific coordinate.
		/// </summary>
		/// <param name="p"> The Point in the ImageGrid</param>
		/// <param name="b"> The Brush used for background</param>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x or y is not within the ImageGrid.</exception>
		public void SetBrush(Point p, Brush b)
		{
			if (p.X<0 || p.X>=NoImagesSize.Width)
				throw new ArgumentOutOfRangeException("p.X", "The x coordinate is \"" + p.X + "\" but has to be in the range [0," + NoImagesSize.Width + "] to be within the ImageGrid");
			if (p.Y<0 || p.Y>=NoImagesSize.Height)
				throw new ArgumentOutOfRangeException("p.Y", "The y coordinate is \"" + p.Y + "\" but has to be in the range [0," + NoImagesSize.Height + "] to be within the ImageGrid");
			Brushes[p.X,p.Y] = b;
		}
		/// <summary>
		/// Get the Brush used as background at a specific coordinate.
		/// </summary>
		/// <param name="p"> The Point in the ImageGrid</param>	
		/// <returns> The Brush used for background</returns>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x or y is not within the ImageGrid.</exception>
		public Brush GetBrush(Point p)
		{
			if (p.X<0 || p.X>=NoImagesSize.Width)
				throw new ArgumentOutOfRangeException("p.X", "The x coordinate is \"" + p.X + "\" but has to be in the range [0," + NoImagesSize.Width + "] to be within the ImageGrid");
			if (p.Y<0 || p.Y>=NoImagesSize.Height)
				throw new ArgumentOutOfRangeException("p.Y", "The y coordinate is \"" + p.Y + "\" but has to be in the range [0," + NoImagesSize.Height + "] to be within the ImageGrid");
			return Brushes[p.X,p.Y];
		}
		/// <summary>
		/// Set the Image used at a specific coordinate.
		/// </summary>
		/// <param name="p"> The Point in the ImageGrid</param>
		/// <param name="i"> The Image used at specified coordinates</param>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x or y is not within the ImageGrid.</exception>
		public void SetImage(Point p, Image i)
		{
			if (p.X<0 || p.X>=NoImagesSize.Width)
				throw new ArgumentOutOfRangeException("p.X", "The x coordinate is \"" + p.X + "\" but has to be in the range [0," + NoImagesSize.Width + "] to be within the ImageGrid");
			if (p.Y<0 || p.Y>=NoImagesSize.Height)
				throw new ArgumentOutOfRangeException("p.Y", "The y coordinate is \"" + p.Y + "\" but has to be in the range [0," + NoImagesSize.Height + "] to be within the ImageGrid");
			if (Images[p.X,p.Y] != i)
			{
				Images[p.X,p.Y] = i;
				this.DrawSquare(p.X,p.Y,null);
			}
		}
		/// <summary>
		/// Get the Image used at a specific coordinate.
		/// </summary>
		/// <param name="p"> The Point in the ImageGrid</param>
		/// <returns> The Image used at specified coordinates</returns>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x or y is not within the ImageGrid.</exception>
		public Image GetImage(Point p)
		{
			if (p.X<0 || p.X>=NoImagesSize.Width)
				throw new ArgumentOutOfRangeException("p.X", "The x coordinate is \"" + p.X + "\" but has to be in the range [0," + NoImagesSize.Width + "] to be within the ImageGrid");
			if (p.Y<0 || p.Y>=NoImagesSize.Height)
				throw new ArgumentOutOfRangeException("p.Y", "The y coordinate is \"" + p.Y + "\" but has to be in the range [0," + NoImagesSize.Height + "] to be within the ImageGrid");
			return Images[p.X,p.Y];
		}

		/// <summary>
		/// Clear all the images from the grid
		/// </summary>
		public void ClearImages()
		{
			for(int x = 0; x<NoImagesSize.Width; x++)
				for(int y = 0; y<NoImagesSize.Height; y++)
				{
					Images[x,y] = null;
				}
		}
		/// <summary>
		/// Set wether the image at a specific coordinate.
		/// </summary>
		/// <param name="p"> The Point in the ImageGrid</param>
		/// <param name="isHighlighted"> If the image at specified coordinates should be highlighted</param>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x or y is not within the ImageGrid.</exception>
		public void SetHighlight(Point p, bool isHighlighted)
		{
			if (p.X<0 || p.X>=NoImagesSize.Width)
				throw new ArgumentOutOfRangeException("p.X", "The x coordinate is \"" + p.X + "\" but has to be in the range [0," + NoImagesSize.Width + "] to be within the ImageGrid");
			if (p.Y<0 || p.Y>=NoImagesSize.Height)
				throw new ArgumentOutOfRangeException("p.Y", "The y coordinate is \"" + p.Y + "\" but has to be in the range [0," + NoImagesSize.Height + "] to be within the ImageGrid");
			this.IsHighlighted[p.X,p.Y] = isHighlighted;
		}
		/// <summary>
		/// Set that no image should be highlighted.
		/// </summary>
		public void SetNoneHighlighted()
		{
			for(int x = 0; x<NoImagesSize.Width; x++)
				for(int y = 0; y<NoImagesSize.Height; y++)
					IsHighlighted[x,y] = false;
			Refresh();
		}

		/// <summary>
		/// Get the Image used at a specific coordinate.
		/// </summary>
		/// <param name="p"> The Point in the ImageGrid</param>
		/// <returns> True if the image at specified coordinates is highlighted</returns>
		/// <exception cref="ArgumentOutOfRangeException">Thrown when x or y is not within the ImageGrid.</exception>
		public bool GetHighlight(Point p)
		{
			if (p.X<0 || p.X>=NoImagesSize.Width)
				throw new ArgumentOutOfRangeException("p.X", "The x coordinate is \"" + p.X + "\" but has to be in the range [0," + NoImagesSize.Width + "] to be within the ImageGrid");
			if (p.Y<0 || p.Y>=NoImagesSize.Height)
				throw new ArgumentOutOfRangeException("p.Y", "The y coordinate is \"" + p.Y + "\" but has to be in the range [0," + NoImagesSize.Height + "] to be within the ImageGrid");
			return this.IsHighlighted[p.X,p.Y];
		}
		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
					LightBrush.Dispose();
					DarkBrush.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// ImageGrid
			// 
			this.Name = "ImageGrid";
			this.Size = new System.Drawing.Size(168, 168);
			this.Click += new System.EventHandler(this.ChessGrid_Click);
			this.Load += new System.EventHandler(this.ChessGrid_Load);
			this.SizeChanged += new System.EventHandler(this.ImageGrid_SizeChanged);
			this.MouseUp += new System.Windows.Forms.MouseEventHandler(this.ChessGrid_MouseUp);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.ChessGrid_Paint);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ImageGrid_MouseDown);

		}
		#endregion

		private void ChessGrid_Load(object sender, System.EventArgs e)
		{
		
		}

		private void DrawSquare(int x, int y, Graphics g)
		{
			Brush CurrentBrush;
			float SquareWidth,SquareHeight, ScreenX, ScreenY;
			float LabelXOff=0,LabelYOff=0;
			if (g==null)
				g = this.CreateGraphics();
			if (HasLabels)
			{
				SquareWidth = Width/((float)NoImagesSize.Width+RelLabelSize);
				SquareHeight = Height/((float)NoImagesSize.Height+RelLabelSize);
				if (LabelAtLeft)
					LabelXOff = SquareWidth*RelLabelSize;					
				else					
					LabelXOff = 0;
				if (LabelAtTop)
					LabelYOff = SquareHeight*RelLabelSize;					
				else
					LabelYOff = 0;
					
			}
			else
			{
				SquareWidth = Width/(float)NoImagesSize.Width;
				SquareHeight = Height/(float)NoImagesSize.Height;
			}
			CurrentBrush = Brushes[x,y];
			if (CurrentBrush == null)
			{
				if ((x+y)%2 == 0)
					CurrentBrush = LightBrush;
				else
					CurrentBrush = DarkBrush;

			}
			ScreenX = LabelXOff + x*SquareWidth;
			ScreenY = LabelYOff + y*SquareHeight;

			g.FillRectangle(CurrentBrush,ScreenX,ScreenY,SquareWidth,SquareHeight);
			if (Images[x,y] != null)
				g.DrawImage(Images[x,y],ScreenX,ScreenY,SquareWidth,SquareHeight);
		
		}

		private void DrawSelectionRectangle(Graphics g)
		{
			if (Selection.X > -1 && Selection.Y > -1)
			{			
				DrawRectangle(Selection.X,Selection.Y,g,SelColor);
			}
		}
		private void DrawRectangle(int X, int Y, Graphics g, Color RectColor)
		{
			float SquareWidth,SquareHeight, ScreenX, ScreenY;
			float LabelXOff=0,LabelYOff=0;
			Pen SelPen;
			if (g==null)
				g = this.CreateGraphics();
			if (HasLabels)
			{
				SquareWidth = Width/((float)NoImagesSize.Width+RelLabelSize);
				SquareHeight = Height/((float)NoImagesSize.Height+RelLabelSize);
				if (LabelAtLeft)
					LabelXOff = SquareWidth*RelLabelSize;					
				else					
					LabelXOff = 0;
				if (LabelAtTop)
					LabelYOff = SquareHeight*RelLabelSize;					
				else
					LabelYOff = 0;
			}
			else
			{
				SquareWidth = Width/(float)NoImagesSize.Width;
				SquareHeight = Height/(float)NoImagesSize.Height;
			}
			ScreenX = LabelXOff + X*SquareWidth;
			ScreenY = LabelYOff + Y*SquareHeight;
			SelPen = new Pen(RectColor,SelWidth*(SquareWidth+SquareHeight)/2);
			g.DrawRectangle(SelPen,ScreenX,ScreenY,SquareWidth,SquareHeight);
			SelPen.Dispose();
		}	
		
		private void DrawHighlightRect(int X, int Y, Graphics g)
		{
			DrawRectangle(X,Y,g,HighlighColor);
		}

		private void DrawGridLabels(Graphics g)
		{
			float SquareWidth,SquareHeight;
			float LabelWidth=0,LabelHeight=0;
			float LabelXOff=0,LabelYOff=0;
			RectangleF rect;
			Font LabelFont;
			StringFormat sf=new StringFormat(StringFormatFlags.DirectionRightToLeft);
			sf.Alignment = StringAlignment.Center;
			sf.LineAlignment = StringAlignment.Center;
			if (!HasLabels)
				return;

			if (g==null)
				g = this.CreateGraphics();
			SquareWidth = Width/((float)NoImagesSize.Width+RelLabelSize);
			SquareHeight = Height/((float)NoImagesSize.Height+RelLabelSize);
			if (LabelAtLeft)
				LabelXOff = SquareWidth*RelLabelSize;					
			else					
				LabelXOff = 0;
			if (LabelAtTop)
				LabelYOff = SquareHeight*RelLabelSize;					
			else
				LabelYOff = 0;
			LabelWidth = SquareWidth*RelLabelSize;
			LabelHeight = SquareHeight*RelLabelSize;
			LabelFont = new Font(this.Font.FontFamily, Math.Min(LabelWidth,LabelHeight)*0.65F);
			//LabelFont = new Font(LabelFontName, Math.Min(LabelWidth,LabelHeight)*0.8F);
			// Draw X labels background
			if (LabelAtTop)
				g.FillRectangle(LabelBackgroundBrush, 0,0,Width,LabelHeight);
			else
				g.FillRectangle(LabelBackgroundBrush, 0,SquareHeight*NoImagesSize.Height,Width,LabelHeight);
			// Draw Y labels background
			if (LabelAtLeft)
				g.FillRectangle(LabelBackgroundBrush, 0,0,LabelWidth,Height);
			else
				g.FillRectangle(LabelBackgroundBrush, SquareWidth*NoImagesSize.Width,0,LabelWidth,Height);

			// Draw X labels
			for(int x=0; x<NoImagesSize.Width; x++)
			{
				if (LabelAtTop)
					rect = new RectangleF(LabelXOff+x*SquareWidth, 0, SquareWidth, LabelHeight);
				else
					rect = new RectangleF(LabelXOff+x*SquareWidth, SquareHeight*NoImagesSize.Height, SquareWidth, LabelHeight);				
				g.DrawString(XLabels[x],LabelFont,LabelBrush,rect,sf);
			}
			// Draw Y labels
			for(int y=0; y<NoImagesSize.Height; y++)
			{
				if (LabelAtLeft)
					rect = new RectangleF(0, LabelYOff+y*SquareHeight, LabelWidth, SquareHeight);
				else
					rect = new RectangleF(SquareWidth*NoImagesSize.Width, LabelYOff+y*SquareHeight, LabelWidth, SquareHeight);
				g.DrawString(YLabels[y],LabelFont,LabelBrush,rect,sf);		
			}
			LabelFont.Dispose();
			sf.Dispose();
		}

		private void ChessGrid_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			//g.DrawString
			DrawGridLabels(e.Graphics);
			for(int x = 0; x<NoImagesSize.Width; x++)
				for(int y = 0; y<NoImagesSize.Height; y++)
				{
					DrawSquare(x,y,e.Graphics);
				}
			DrawSelectionRectangle(e.Graphics);
			for(int x = 0; x<NoImagesSize.Width; x++)
				for(int y = 0; y<NoImagesSize.Height; y++)
				{
					if (this.IsHighlighted[x,y])
						DrawHighlightRect(x,y,e.Graphics);
				}
		}

		private void ChessGrid_Click(object sender, System.EventArgs e)
		{
		
		}

		private void OnSelect()
		{
			if (Selected != null)
			{
				if (Selection.X > -1 && Selection.Y > -1)
					Selected(this, Selection);
			}
		}

		private void OnUnSelect()
		{
			if (Selection.X >= 0 && Selection.Y >= 0)
			{
				if (UnSelected != null)
				{	
					if (Selection.X > -1 && Selection.Y > -1)
						UnSelected(this, Selection);
				}
				Selection.X = -1;
				Selection.Y = -1;
			}
		}

		private Point ConvertScreenToImagePoint(int x, int y)
		{
			float SquareWidth,SquareHeight;
			float LabelXOff=0,LabelYOff=0;
			SquareWidth = Width/((float)NoImagesSize.Width+RelLabelSize);
			SquareHeight = Height/((float)NoImagesSize.Height+RelLabelSize);
			if (LabelAtLeft)
				LabelXOff = SquareWidth*RelLabelSize;					
			else					
				LabelXOff = 0;
			if (LabelAtTop)
				LabelYOff = SquareHeight*RelLabelSize;					
			else
				LabelYOff = 0;
			Point p = new Point();
			p.Y = (int)((y-LabelYOff)/SquareHeight);
			if (p.Y>=NoImagesSize.Height)
				p.Y = NoImagesSize.Height-1;
			p.X = (int)((x-LabelXOff)/SquareWidth);
			if (p.X>=NoImagesSize.Width)
				p.X = NoImagesSize.Width-1;
			return p;
		}

		/// <summary>
		/// Unselect the currently selected image.
		/// </summary>		
		public void DoUnselect()
		{
			OnUnSelect();
			/*Refresh();*/
		}

		private void ChessGrid_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			Point p, SelBackup;
			p = ConvertScreenToImagePoint(e.X,e.Y);
			if (e.Button == MouseButtons.Left && MouseDownPoint.X >=0 && MouseDownPoint.Y >= 0)
			{
				if (p == MouseDownPoint) // "Click"
				{
					if (Selection.X >= 0 && Selection.Y >= 0) //Exist a selection
					{
						if (Selection == p) // Click on the same square => deselection
						{
							OnUnSelect();
							if (SquareClick != null)
								SquareClick(this, p);
						}
						else // Clicked on new square
						{
							SelBackup = Selection;
							OnUnSelect();
							if (SquareClick != null)
								SquareClick(this, p);
							Selection = p;
							OnSelect();
							if (SelectionChanged != null)
								SelectionChanged(this,SelBackup,p);			
						}
						Refresh();
					}
					else // Exist no selection, First Click
					{
						if (SquareClick != null)
							SquareClick(this, p);
						Selection = p;
						OnSelect();
						Refresh();
					}
				}
				else // Drag
				{
					OnUnSelect();					
					if (Dragged != null)
						Dragged(this, MouseDownPoint, p);
					MouseDownPoint = new Point(-1,-1);
				}
			}
		}

		private void ImageGrid_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			switch(e.Button) 
			{
				case MouseButtons.Right:					
					OnUnSelect();
					Refresh();
					MouseDownPoint = new Point(-1,-1);
					break;
				case MouseButtons.Left:
					MouseDownPoint = ConvertScreenToImagePoint(e.X,e.Y);
					break;
			}
		}

		private void ImageGrid_SizeChanged(object sender, System.EventArgs e)
		{
			this.Refresh();
		}
	}
}
