using System;
using System.Collections;
using System.Xml;

namespace ChessApplication
{
	/// <summary>
	/// Class with the behavior of the King.
	/// </summary>
	public class King : ChessPiece
	{
		/// <summary>
		/// Constructor of a King.
		/// </summary>
		/// <param name="color"> The color or side of the Piece</param>
		/// <param name="board"> Reference to the ChessBoard this Piece is placed on</param>
		/// <param name="position"> The position on the board</param>
		/// <param name="relativeValue">The relative value of the Piece</param>
		/// <param name="movementBonus">The bonus amount for the Piece</param>/// 
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public King(ChessPieceColor color, ChessBoard board, ChessPosition position, double relativeValue, double movementBonus) 
			: base(color,board,position,relativeValue,movementBonus) { }
		/// <summary>
		/// Deserialization constructor of a position on the Chessboard.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		public King(XmlReader reader, ChessBoard board) 
			: base(reader,board) { }
		/// <summary>
		/// Get the total number of possible movement for this type of a piece.
		/// </summary>
		/// <returns> Returns possible movement</returns>
		public override int GetAbsoluteMovability()
		{
			return 8;
		}
		private void KingMove(ChessPosition newPosition, Queue q)
		{
			if (ChessBoard.IsInsideBoard(newPosition))
			{
				if (board.IsSqareOccupied(newPosition))
				{
					if (HasOppositeColor(newPosition))
						q.Enqueue(new ChessMove(position,newPosition));
						//if (!board.IsPositionCheckedBy(newPosition, GetOppositeColor()))						
				}
				else
				{
					//if (!board.IsPositionCheckedBy(newPosition, GetOppositeColor()))
					q.Enqueue(new ChessMove(position, newPosition));
				}
			}
		}

		private void Castling(Queue q)
		{
			ChessPosition RookPosition,step1,step2,step3;
			ChessPiece RookPiece;
			// Short Castling
			if (color == ChessPieceColor.Black)
				RookPosition = new ChessPosition(ChessPosition.Column_H,8);
			else
				RookPosition = new ChessPosition(ChessPosition.Column_H,1);
			RookPiece = board.GetPiece(RookPosition);
			if (!HasMoved && (RookPiece != null && !RookPiece.HasMoved))
			{
				step1 = position.WithAddition(+1,0);
				step2 = position.WithAddition(+2,0);
				if (!board.IsSqareOccupied(step1) && !board.IsSqareOccupied(step2))
				{
					if (!board.IsPositionCheckedBy(position,GetOppositeColor())
						&& !board.IsPositionCheckedBy(step1,GetOppositeColor())
						&& !board.IsPositionCheckedBy(step2,GetOppositeColor()))
					{
						q.Enqueue(new ChessMove(position,step2));
					}
				}
			}
			// Long Castling
			if (color == ChessPieceColor.Black)
				RookPosition = new ChessPosition(ChessPosition.Column_A,8);
			else
				RookPosition = new ChessPosition(ChessPosition.Column_A,1);
			RookPiece = board.GetPiece(RookPosition);
			if (!HasMoved && (RookPiece != null && !RookPiece.HasMoved))
			{
				step1 = position.WithAddition(-1,0);
				step2 = position.WithAddition(-2,0);
				step3 = position.WithAddition(-3,0);
				if (!board.IsSqareOccupied(step1) && !board.IsSqareOccupied(step2) && !board.IsSqareOccupied(step3))
				{
					if (!board.IsPositionCheckedBy(position,GetOppositeColor())
						&& !board.IsPositionCheckedBy(step1,GetOppositeColor())
						&& !board.IsPositionCheckedBy(step2,GetOppositeColor()))
					{
						q.Enqueue(new ChessMove(position,step2));
					}
				}
			}
		}

		/// <summary>
		/// Determine if the King is Checked by a opponent.
		/// </summary>
		/// <returns> Returns true if the King is Checked by a opponent</returns>
		public bool IsChecked()
		{
			return board.IsPositionCheckedBy(position, GetOppositeColor());
		}
		/// <summary>
		/// Generate all the possible legal moves for this except castling.		
		/// </summary>
		/// <param name="q"> The Queue to add the move to</param>
		/// <exception cref="ArgumentException">Thrown when the Queue q is null (not created)</exception>
		public void AddLegalMovesNoCastling(Queue q)
		{
			//Moving right
			KingMove(position.WithAddition(+1,0),q);
			//Moving right-up
			KingMove(position.WithAddition(+1,+1),q);
			//Moving up
			KingMove(position.WithAddition(0,+1),q);
			//Moving left-up
			KingMove(position.WithAddition(-1,+1),q);
			//Moving left
			KingMove(position.WithAddition(-1,0),q);
			//Moving left-down
			KingMove(position.WithAddition(-1,-1),q);
			//Moving down
			KingMove(position.WithAddition(0,-1),q);
			//Moving right-down
			KingMove(position.WithAddition(+1,-1),q);
		}
		/// <summary>
		/// Generate all the possible legal moves for this piece at current state of the chessboard.		
		/// </summary>
		/// <param name="q"> The Queue to add the move to</param>
		/// <exception cref="ArgumentException">Thrown when the Queue q is null (not created)</exception>
		public override void AddLegalMoves(Queue q)
		{
			AddLegalMovesNoCastling(q);
			// Castling
			Castling(q);
		}
		/// <summary>
		/// Move the King to another position without verify that it is a correct move.
		/// </summary>
		/// <param name="newPosition"> The position to move to</param>
		public override void MoveToNotChecked(ChessPosition newPosition)
		{
			ChessPosition shortCastlingPosition,longCastlingPosition;
			ChessPosition rookFrom, rookTo, kingTo;
			ChessPiece rookPiece;
			shortCastlingPosition = position.WithAddition(+2,0);
			longCastlingPosition = position.WithAddition(-2,0);
			if (newPosition == shortCastlingPosition)
			{
				rookFrom = position.WithAddition(+3,0);
				rookTo = position.WithAddition(+1,0);
				kingTo = position.WithAddition(+2,0);
				rookPiece = board.GetPiece(rookFrom);
				rookPiece.MoveToNotChecked(rookTo);
			}
			else if (newPosition == longCastlingPosition)
			{
				rookFrom = position.WithAddition(-4,0);
				rookTo = position.WithAddition(-1,0);
				kingTo = position.WithAddition(-2,0);
				rookPiece = board.GetPiece(rookFrom);
				rookPiece.MoveToNotChecked(rookTo);
			}
			if (board.IsSqareOccupied(newPosition))
				board.SetCaptureMoveCurrent();
			board.SetPiece(newPosition, this);
			board.SetPiece(position, null);
			position = newPosition;
			lastMove = board.NumberOfMovePlayed;
		}
		/// <summary>
		/// Makes a good string of the state
		/// </summary>
		public override string ToString()
		{
			if (color == ChessPieceColor.White)
				return "King<White,," + position + ">";
			else
				return "King<Black,," + position + ">";
		}
	}
}