using System;
using System.Collections;
using System.Xml;

namespace ChessApplication
{
	/// <summary>
	/// Class with the behavior of the Knight.
	/// </summary>
	[Serializable()]
	public class Knight : ChessPiece
	{
		/// <summary>
		/// Constructor of a Knight.
		/// </summary>
		/// <param name="color"> The color or side of the Piece</param>
		/// <param name="board"> Reference to the ChessBoard this Piece is placed on</param>
        /// <param name="position"> The position on the board</param>
		/// <param name="relativeValue">The relative value of the Piece</param>
		/// <param name="movementBonus">The bonus amount for the Piece</param>/// 
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public Knight(ChessPieceColor color, ChessBoard board, ChessPosition position, double relativeValue, double movementBonus) 
			: base(color,board,position,relativeValue,movementBonus) { }
		/// <summary>
		/// Deserialization constructor of a position on the Chessboard.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		public Knight(XmlReader reader, ChessBoard board) 
			: base(reader,board) { }
		/// <summary>
		/// Get the total number of possible movement for this type of a piece.
		/// </summary>
		/// <returns> Returns possible movement</returns>
		public override int GetAbsoluteMovability()
		{
			return 8;
		}
		private void KnightMove(ChessPosition newPosition, Queue q, ChessMoveType mtype)
		{
			if (ChessBoard.IsInsideBoard(newPosition))
			{
				if (board.IsSqareOccupied(newPosition))
				{
					if (HasOppositeColor(newPosition))
						q.Enqueue(new ChessMove(position, newPosition, ChessMoveType.Capture));
				}
				else
				{
					q.Enqueue(new ChessMove(position, newPosition, mtype));
				}
			}
		}
		/// <summary>
		/// Generate all the possible legal moves for this piece at current state of the chessboard.		
		/// </summary>
		/// <param name="q"> The Queue to add the move to</param>
		/// <exception cref="ArgumentException">Thrown when the Queue q is null (not created)</exception>
		public override void AddLegalMoves(Queue q)
		{
			ChessMoveType mtype;
			if (q==null)
				throw new ArgumentException("q", "The queue is not created");
			if (color == ChessPieceColor.White)
				mtype = ChessMoveType.Forward;
			else
				mtype = ChessMoveType.Backward;
			KnightMove(position.WithAddition(+1,+2),q,mtype);
			KnightMove(position.WithAddition(+2,+1),q,mtype);
			KnightMove(position.WithAddition(-2,+1),q,mtype);
			KnightMove(position.WithAddition(-1,+2),q,mtype);
			if (color == ChessPieceColor.White)
				mtype = ChessMoveType.Backward;					
			else
				mtype = ChessMoveType.Forward;
			KnightMove(position.WithAddition(+2,-1),q,mtype);
			KnightMove(position.WithAddition(+1,-2),q,mtype);
			KnightMove(position.WithAddition(-1,-2),q,mtype);
			KnightMove(position.WithAddition(-2,-1),q,mtype);
		}
		/// <summary>
		/// Makes a good string of the state
		/// </summary>
		public override string ToString()
		{
			if (color == ChessPieceColor.White)
				return "Knight<White,," + position + ">";
			else
				return "Knight<Black,," + position + ">";
		}
	}
}
