using System;
using System.Collections;
using System.Xml;
using System.Windows.Forms;
using System.IO;

namespace ChessApplication
{
	/// <summary>
	/// Class with the behavior of the Pawn.
	/// </summary>
	class Pawn : ChessPiece 
	{
		/// <summary>
		/// Constructor of a Pawn.
		/// </summary>
		/// <param name="color"> The color or side of the Piece</param>
		/// <param name="board"> Reference to the ChessBoard this Piece is placed on</param>
        /// <param name="position"> The position on the board</param>
		/// <param name="relativeValue">The relative value of the Piece</param>
		/// <param name="movementBonus">The bonus amount for the Piece</param>/// 
		/// <exception cref="ArgumentOutOfRangeException">Thrown when row or column is not within the Chessboard.</exception>
		public Pawn(ChessPieceColor color, ChessBoard board, ChessPosition position, double relativeValue, double movementBonus) 
			: base(color,board,position,relativeValue,movementBonus) { }
		/// <summary>
		/// Deserialization constructor of a position on the Chessboard.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		public Pawn(XmlReader reader, ChessBoard board) 
			: base(reader,board) { }
/*
		/// <summary>The relative value of the Piece.</summary>
		public override double GetRelativeValue()
		{
			if (color == ChessPieceColor.Black)
			{
				return relativeValue*(1.0 + (9-position.Row)/8.0);
			}
			else
			{
				return relativeValue*(1.0 + position.Row/8.0);
			}
		}
		*/
		/// <summary>
		/// Move the Pawn to another position without verify that it is a correct move.
		/// </summary>
		/// <param name="newPosition"> The position to move to</param>
		public override void MoveToNotChecked(ChessPosition newPosition)
		{
			ChessPiece toSet;
			ChessPosition EnPassantCapture;
			// Account for en-passant taking
			if (color == ChessPieceColor.White) 
			{
				if (position.Row==5 && newPosition.Row==6)
				{
					if ((position.Column+1) == newPosition.Column || (position.Column-1) == newPosition.Column)
					{
						EnPassantCapture = new ChessPosition(newPosition.Column,position.Row);
						if (!board.IsSqareOccupied(newPosition)
							&& board.IsSqareOccupied(EnPassantCapture)
							&& ((board.GetPiece(EnPassantCapture)) is Pawn)
							&& HasOppositeColor(EnPassantCapture))
						{
							board.SetPiece(EnPassantCapture, null);
						}	
					}
				}
			}
			else
			{
				if (position.Row==4 && newPosition.Row==3)
				{
					if ((position.Column+1) == newPosition.Column || (position.Column-1) == newPosition.Column)
					{
						EnPassantCapture = new ChessPosition(newPosition.Column,position.Row);
						if (!board.IsSqareOccupied(newPosition)
							&& board.IsSqareOccupied(EnPassantCapture)
							&& ((board.GetPiece(EnPassantCapture)) is Pawn)
							&& HasOppositeColor(EnPassantCapture))
						{
							board.SetPiece(EnPassantCapture, null);
						}	
					}
				}
			}
			// End - Account for en-passant taking
			toSet = this;
			if (IsColorWhite())
			{
				if (newPosition.Row==8)
				{
					//Create Queen (Changing the pawn into a queen)
					toSet = new Queen(color,board,newPosition,board.GetQueenRelativeValue(),board.GetQueenMovementBonus());
				} 
			}
			else
			{
				if (newPosition.Row==1)
				{
					//Create Queen (Changing the pawn into a queen)
					toSet = new Queen(color,board,newPosition,board.GetQueenRelativeValue(),board.GetQueenMovementBonus());
				}
			}
			if (board.IsSqareOccupied(newPosition))
				board.SetCaptureMoveCurrent();
			board.SetPawnMoveCurrent();
			board.SetPiece(position, null);
			board.SetPiece(newPosition, toSet);
			position = newPosition;
			lastMove = board.NumberOfMovePlayed;
		}
		/// <summary>
		/// Get the total number of possible movement for this type of a piece.
		/// </summary>
		/// <returns> Returns possible movement</returns>
		public override int GetAbsoluteMovability()
		{
			if (IsColorWhite())
			{
				if (position.Row==2)
					return 2;
				else
					return 1;
			}
			else
			{
				if (position.Row==7)
					return 2;
				else
					return 1;
			}
		}
		/// <summary>
		/// Generate all the possible legal moves for this piece at current state of the chessboard.		
		/// </summary>
		/// <param name="q"> The Queue to add the move to</param>
		/// <exception cref="ArgumentException">Thrown when the Queue q is null (not created)</exception>
		public override void AddLegalMoves(Queue q)
		{
			ChessPosition EnPassantCapture;
			if (q==null)
				throw new ArgumentException("q", "The queue is not created");
			ChessPosition newPosition;
			if (IsColorWhite())
			{
				// Ordinary forward move
				newPosition = position.WithAddition(0,+1);
				if (ChessBoard.IsInsideBoard(newPosition) && !board.IsSqareOccupied(newPosition))
				{
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.ForwardPlus));
				}
				// Taking piece diagonally right
				newPosition = position.WithAddition(+1,+1);
				if (ChessBoard.IsInsideBoard(newPosition)
					&& board.IsSqareOccupied(newPosition)
					&& HasOppositeColor(newPosition))
				{
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.Capture));
				}
				// Taking piece diagonally left
				newPosition = position.WithAddition(-1,+1);
				if (ChessBoard.IsInsideBoard(newPosition)
					&& board.IsSqareOccupied(newPosition) 
					&& HasOppositeColor(newPosition))
				{				
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.Capture));
				}
				// Double forward move (the first pawn move)
				newPosition = position.WithAddition(0,+2);
				if (position.Row==2 
					&& !board.IsSqareOccupied(position.WithAddition(0,+1))
					&& !board.IsSqareOccupied(newPosition))
				{				
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.ForwardPlus2));
				}
				// en-passant taking piece diagonally right
				newPosition = position.WithAddition(+1,+1);
				EnPassantCapture = position.WithAddition(+1,0);
				if (position.Row==5 
					&& ChessBoard.IsInsideBoard(newPosition)
					&& !board.IsSqareOccupied(newPosition)
					&& board.IsSqareOccupied(EnPassantCapture)
					&& (board.GetPiece(EnPassantCapture)) is Pawn
					&& board.GetPiece(EnPassantCapture).WhenLastMove == (board.NumberOfMovePlayed-2)
					&& HasOppositeColor(EnPassantCapture))
				{
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.Capture));
				}
				// en-passant taking piece diagonally left
				newPosition = position.WithAddition(-1,+1);
				EnPassantCapture = position.WithAddition(-1,0);
				if (position.Row==5 
					&& ChessBoard.IsInsideBoard(newPosition)
					&& !board.IsSqareOccupied(newPosition)
					&& board.IsSqareOccupied(EnPassantCapture)
					&& (board.GetPiece(EnPassantCapture)) is Pawn
					&& board.GetPiece(EnPassantCapture).WhenLastMove == (board.NumberOfMovePlayed-2)
					&& HasOppositeColor(EnPassantCapture))
				{
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.Capture));
				}
			}
			else // Blacks move
			{
				// Ordinary forward move
				newPosition = position.WithAddition(0,-1);
				if (ChessBoard.IsInsideBoard(newPosition) && !board.IsSqareOccupied(newPosition))
				{
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.ForwardPlus));
				}
				// Taking piece diagonally left
				newPosition = position.WithAddition(+1,-1);
				if (ChessBoard.IsInsideBoard(newPosition)
					&& board.IsSqareOccupied(newPosition)
					&& HasOppositeColor(newPosition))
				{
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.Capture));
				}
				// Taking piece diagonally right
				newPosition = position.WithAddition(-1,-1);
				if (ChessBoard.IsInsideBoard(newPosition)
					&& board.IsSqareOccupied(newPosition) 
					&& HasOppositeColor(newPosition))
				{				
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.Capture));
				}
				// Double forward move (the first pawn move)
				newPosition = position.WithAddition(0,-2);
				if (position.Row==7 
					&& !board.IsSqareOccupied(position.WithAddition(0,-1))
					&& !board.IsSqareOccupied(newPosition))
				{				
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.ForwardPlus2));
				}
				// en-passant taking piece diagonally left
				newPosition = position.WithAddition(+1,-1);
				EnPassantCapture = position.WithAddition(+1,0);
				if (position.Row==4 
					&& ChessBoard.IsInsideBoard(newPosition)
					&& !board.IsSqareOccupied(newPosition)
					&& board.IsSqareOccupied(EnPassantCapture)
					&& (board.GetPiece(EnPassantCapture)) is Pawn
					&& board.GetPiece(EnPassantCapture).WhenLastMove == (board.NumberOfMovePlayed - 2)
					&& HasOppositeColor(EnPassantCapture))
				{
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.Capture));
				}
				// en-passant taking piece diagonally right
				newPosition = position.WithAddition(-1,-1);
				EnPassantCapture = position.WithAddition(-1,0);
				if (position.Row==4 
					&& ChessBoard.IsInsideBoard(newPosition)
					&& !board.IsSqareOccupied(newPosition)
					&& board.IsSqareOccupied(EnPassantCapture)
					&& (board.GetPiece(EnPassantCapture)) is Pawn
					&& board.GetPiece(EnPassantCapture).WhenLastMove == (board.NumberOfMovePlayed - 2)
					&& HasOppositeColor(EnPassantCapture))
				{
					q.Enqueue(new ChessMove(position,newPosition,ChessMoveType.Capture));
				}
			}
		}
		/// <summary>
		/// Makes a good string of the state
		/// </summary>
		public override string ToString()
		{
			if (color == ChessPieceColor.White)
                return "Pawn<White,," + position + ">";
			else
				return "Pawn<Black,," + position + ">";
		}
	}
}
