using System;
using System.Xml;
using System.Collections;

namespace ChessApplication
{
	/// <summary>
	/// The simple engine for generating moves for a chessboard using
	/// search on a number(N) of the most promising nodes.
	/// </summary>
	public class SearchNBestEngine : ChessEngine
	{
		/// <summary>
		/// The number of the best nodes to expand.
		/// </summary>
		protected int nrNodeToExpand;

		static private double epsilon=0.1;
		/// <summary>
		/// Constructor of a SearchNBestEngine.
		/// </summary>
		/// <param name="board"> The board to generate moves for.</param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		public SearchNBestEngine(ChessBoard board, Stack boardHistoryStack)
			: base(board,boardHistoryStack,4)
		{ 
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove); 
			this.nrNodeToExpand = 6;
		}
		/// <summary>
		/// Constructor of a SearchNBestEngine.
		/// </summary>
		/// <param name="board"> The board to generate moves for</param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		/// <param name="searchDepth"> The depth of the search</param>
		/// <param name="nrNodeToExpand"> The number of best nodes to expand</param>
		public SearchNBestEngine(ChessBoard board, Stack boardHistoryStack, int searchDepth, int nrNodeToExpand)
			: base(board,boardHistoryStack, searchDepth) 
		{ 
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove);
			this.nrNodeToExpand = nrNodeToExpand;
		}
		/// <summary>
		/// Deserialization constructor of a SearchNBestEngine.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		public SearchNBestEngine(XmlReader reader, ChessBoard board, Stack boardHistoryStack) 
			: base(board,boardHistoryStack)
		{
			this.forBoard = board;
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove);
			reader.ReadStartElement();
			while (true)
				if (reader.IsStartElement("searchDepth")) 
				{
					searchDepth = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if (reader.IsStartElement("nrNodeToExpand")) 
				{
					nrNodeToExpand = XmlConvert.ToInt32(reader.ReadString());
					reader.ReadEndElement();
				}
				else if ((reader.NodeType == XmlNodeType.EndElement) && (reader.Name == this.GetType().Name)) 
				{
					reader.ReadEndElement();
					break;
				}
				else 
				{
					reader.Skip();
					//throw new ApplicationException("Unknown node found");
				}
		}
		/// <summary>
		/// The approximate number of nodes the have to be searched
		/// to decide the best move.
		/// </summary>
		/// <returns> Returns the number of nodes that will be searched</returns>
		public override int ApproxNrNodes()
		{
			return 35*(int)Math.Pow(nrNodeToExpand,searchDepth-1);
		}
		
		/// <summary>
		/// Decides the best move and returns it.
		/// </summary>
		/// <returns> The best move using this engine</returns>
		public override ChessMove DecideBestMove()
		{
			ChessMove[] moves;
			ChessBoard[] boards;
			double[] values;
			nrGeneratedMoves = 0;
			forBoard.GenerateSucessor(out moves, out boards);
			nrGeneratedMoves += moves.Length;
			values = new double[boards.Length];
			boardHistoryStack.Push(forBoard);
			for(int i=0; i<boards.Length; ++i)
				values[i] = GenerateMoveValue(boards[i],1);
			Array.Sort(values,moves);
			nrGeneratedMoves = 0;
			boardHistoryStack.Pop();
			if (values[values.Length-1]<0.0)
			{
				if (forBoard.Draw50MovesRule())
					return ChessMove.ClaimDrawRule50Move();
				if (forBoard.DrawMoveRepetition(boardHistoryStack))
					return ChessMove.ClaimDrawRuleRepetition();
			}
			return moves[values.Length-1];
		}

		private void SelectBest(ChessBoard[] boards, double[] values, out ChessBoard[] selBoards, out double[] selValues)
		{
			double startValue;
			int nrSame;
			if (boards.Length <= nrNodeToExpand)
			{
				selBoards = boards;
				selValues = values;
			}
			else
			{		
				selBoards = new ChessBoard[nrNodeToExpand];
				selValues = new double[nrNodeToExpand];
				startValue = values[0];
				nrSame = 1;
				for(int i=1; i<boards.Length; ++i)
				{
					if (Math.Abs(values[i]-startValue)>epsilon)
						break;
					nrSame++;
				}
				if (nrSame <= nrNodeToExpand)
				{
					Array.Copy(boards,selBoards,nrNodeToExpand);
					Array.Copy(values,selValues,nrNodeToExpand);
				}
				else
				{
					Random rg=new Random();
					int randNr;
					for(int i=nrNodeToExpand; i>0; --i)
					{
						randNr = rg.Next(i);
						selBoards[i-1] = boards[randNr];
						boards[randNr] = boards[i-1];
						selValues[i-1] = values[randNr];
						values[randNr] = values[i-1];
					}
				}
			}
		}


		private double GenerateMoveValue(ChessBoard board, int level)
		{
			ChessMove[] moves;
			ChessBoard[] boards, oldBoards;
			double[] values,oldValues;
			double retValue;
			board.GenerateSucessor(out moves, out oldBoards);
			if (TerminalCondition(moves,board,level,out retValue))
				return retValue;
			oldValues = this.GenerateBoardValues(oldBoards,forBoard.ColorToMove);
			Array.Sort(oldValues,oldBoards);
			if (forBoard.ColorToMove == board.ColorToMove)
			{
				Array.Reverse(oldValues);
				Array.Reverse(oldBoards);
			}
			SelectBest(oldBoards,oldValues,out boards,out values);
			nrGeneratedMoves += boards.Length;
			if (level < (searchDepth-1))
			{
				for(int i=0; i<boards.Length; ++i)
					values[i] = GenerateMoveValue(boards[i],level+1);
			}
            Array.Sort(values);
			if (forBoard.ColorToMove == board.ColorToMove)
			{
				return FilterForVoluntaryDrawCondition(board,values[values.Length-1]);
				//return values[values.Length-1];
			}
			else
			{
				return FilterForVoluntaryDrawCondition(board,values[0]);
				//return values[0];
			}
		}
		/// <summary>
		/// Xml saving function.
		/// </summary>
		/// <param name="writer"> The Xml writer</param>
		public override void SaveToXml(XmlWriter writer)
		{
			writer.WriteStartElement(this.GetType().Name);
			writer.WriteElementString("searchDepth",XmlConvert.ToString(searchDepth));
			writer.WriteElementString("nrNodeToExpand",XmlConvert.ToString(nrNodeToExpand));
			writer.WriteEndElement();
		}

	}
}
