using System;
using System.Xml;
using System.Collections;

namespace ChessApplication
{
	/// <summary>
	/// The simple engine for generating moves for a chessboard using
	/// exhaustive search. Ineffective but simple to implement.
	/// </summary>
	public class SimpleEngine : ChessEngine
	{
		/// <summary>
		/// Constructor of a SimpleEngine.
		/// </summary>
		/// <param name="board"> The board to generate moves for.</param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		public SimpleEngine(ChessBoard board, Stack boardHistoryStack)
			: base(board,boardHistoryStack) 
		{ 
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove); 
		}
		/// <summary>
		/// Constructor of a SimpleEngine.
		/// </summary>
		/// <param name="board"> The board to generate moves for</param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		/// <param name="searchDepth"> The depth of the search</param>
		public SimpleEngine(ChessBoard board, Stack boardHistoryStack, int searchDepth)
			: base(board, boardHistoryStack, searchDepth) 
		{ 
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove); 
		}
		/// <summary>
		/// Deserialization constructor of a ChessEngine.
		/// </summary>
		/// <param name="reader"> The XmlReader</param>				
		/// <param name="board"> The board reference </param>
		/// <param name="boardHistoryStack"> The history of the chess game i.e. old ChessBoards</param>
		public SimpleEngine(XmlReader reader, ChessBoard board, Stack boardHistoryStack) 
			: base(reader,board,boardHistoryStack) 
		{ 
			DecideBestMoveCaller = new DecideBestMoveDelegate(this.DecideBestMove); 
		}
		/// <summary>
		/// Decides the best move and returns it.
		/// </summary>
		/// <returns> The best move using this engine</returns>
/*		public override ChessMove DecideBestMove()
		{
			ChessMove[] moves;
			ChessBoard[] boards;
			double[] values;
			nrGeneratedMoves = 0;
			forBoard.GenerateSucessor(out moves, out boards);
			Array.Sort(moves,boards);
			nrGeneratedMoves += moves.Length;
			values = new double[boards.Length];
			for(int i=0; i<boards.Length; ++i)
				values[i] = GenerateMoveValue(boards[i],1);
			Array.Sort(values,moves);
			nrGeneratedMoves = 0;
			if (values[values.Length-1]<0.0)
			{
				if (forBoard.Draw50MovesRule())
					return ChessMove.ClaimDrawRule50Move();
				if (forBoard.DrawMoveRepetition(boardHistoryStack))
					return ChessMove.ClaimDrawRuleRepetition();
			}
			return moves[values.Length-1];
		}
		*/
		public override ChessMove DecideBestMove()
		{
			ChessMove[] moves;
			ChessBoard[] boards;
			double[] values;
			double bestValue=-2000;
			double currentValue;
			ChessMove bestMove = null;
			nrGeneratedMoves = 0;
			forBoard.GenerateSucessor(out moves, out boards);
			Array.Sort(moves,boards);
			nrGeneratedMoves += moves.Length;
			values = new double[boards.Length];
			boardHistoryStack.Push(forBoard);
			for(int i=0; i<boards.Length; ++i)
			{
				currentValue = GenerateMoveValue(boards[i],1);
				if (currentValue > bestValue)
				{
					bestValue = currentValue;
					bestMove = moves[i];
				}
			}
			nrGeneratedMoves = 0;
			boardHistoryStack.Pop();
			if (bestValue<0.0)
			{
				if (forBoard.Draw50MovesRule())
					return ChessMove.ClaimDrawRule50Move();
				if (forBoard.DrawMoveRepetition(boardHistoryStack))
					return ChessMove.ClaimDrawRuleRepetition();
			}
			return bestMove;
		}
/*
		public override ChessMove DecideBestMove()
		{
			ChessMove[] moves;
			ChessBoard[] boards;
			double[] values;
			nrGeneratedMoves = 0;
			forBoard.GenerateSucessor(out moves, out boards);
			nrGeneratedMoves += moves.Length;
			values = new double[boards.Length];
			for(int i=0; i<boards.Length; ++i)
				values[i] = GenerateMoveValue(boards[i],1);
			Array.Sort(values,moves);
			nrGeneratedMoves = 0;
			if (values[values.Length-1]<0.0)
			{
				if (forBoard.Draw50MovesRule())
					return ChessMove.ClaimDrawRule50Move();
				if (forBoard.DrawMoveRepetition(boardHistoryStack))
					return ChessMove.ClaimDrawRuleRepetition();
			}
			return moves[values.Length-1];
		}*/

		private double GenerateMoveValue(ChessBoard board, int level)
		{
			ChessMove[] moves;
			ChessBoard[] boards;
			double minValue=2*1000;
			double maxValue=2*-1000;
			double retValue;
			if (level == searchDepth)
			{				
				return board.GenerateBoardValue(forBoard.ColorToMove);
			}
			else
			{				
				board.GenerateSucessor(out moves, out boards);
				nrGeneratedMoves += moves.Length;
				if (TerminalCondition(moves,board,level,out retValue))
					return retValue;
				if (forBoard.ColorToMove == board.ColorToMove)
				{
					boardHistoryStack.Push(board);
					for(int i=0; i<boards.Length; ++i)
						maxValue = Math.Max(maxValue,GenerateMoveValue(boards[i],level+1));
					boardHistoryStack.Pop();
					return FilterForVoluntaryDrawCondition(board,maxValue);
				    //return maxValue;
				}
				else
				{
					boardHistoryStack.Push(board);
					for(int i=0; i<boards.Length; ++i)
						minValue = Math.Min(minValue,GenerateMoveValue(boards[i],level+1));
					boardHistoryStack.Pop();
					return FilterForVoluntaryDrawCondition(board,minValue);
					//return minValue;
				}
			}
		}
/*
		private double GenerateMoveValue(ChessBoard board, int level)
		{
			ChessMove[] moves;
			ChessBoard[] boards;
			double[] values;
			if (level == searchDepth)
			{				
				return board.GenerateBoardValue(forBoard.ColorToMove);
			}
			else
			{				
				board.GenerateSucessor(out moves, out boards);
				nrGeneratedMoves += moves.Length;
				values = new double[boards.Length];
				for(int i=0; i<boards.Length; ++i)
					values[i] = GenerateMoveValue(boards[i],level+1);
				Array.Sort(values);
				//return values[values.Length-1];
				if (forBoard.ColorToMove == board.ColorToMove)
				{
					if (values.Length >= 1)
						return values[values.Length-1];							
					else
						return 2*1000;
				}
				else
				{
					if (values.Length >=1)
						return values[0];
					else
						return 2*-1000;
				}
			}
		}*/
	}
}
